@@ -30,33 +30,93 @@ use jj_lib::revset::{
     RevsetEvaluationError, RevsetParseError, RevsetParseErrorKind, RevsetResolutionError,
 };
 use jj_lib::signing::SignInitError;
-use jj_lib::tree::TreeMergeError;
+use jj_lib::str_util::StringPatternParseError;
 use jj_lib::working_copy::{ResetError, SnapshotError, WorkingCopyStateError};
 use jj_lib::workspace::WorkspaceInitError;
 use thiserror::Error;
 
+use crate::formatter::{FormatRecorder, Formatter};
 use crate::merge_tools::{
     ConflictResolveError, DiffEditError, DiffGenerateError, MergeToolConfigError,
 };
+use crate::revset_util::UserRevsetEvaluationError;
 use crate::template_parser::{TemplateParseError, TemplateParseErrorKind};
 use crate::ui::Ui;
 
-#[derive(Clone, Debug)]
-pub enum CommandError {
-    UserError {
-        err: Arc<dyn error::Error + Send + Sync>,
-        hint: Option<String>,
-    },
-    ConfigError(String),
-    /// Invalid command line
-    CliError(String),
-    /// Invalid command line detected by clap
-    ClapCliError {
-        err: Arc<clap::Error>,
-        hint: Option<String>,
-    },
+#[derive(Clone, Copy, Debug, Eq, PartialEq)]
+pub enum CommandErrorKind {
+    User,
+    Config,
+    /// Invalid command line. The inner error type may be `clap::Error`.
+    Cli,
     BrokenPipe,
-    InternalError(Arc<dyn error::Error + Send + Sync>),
+    Internal,
+}
+
+#[derive(Clone, Debug)]
+pub struct CommandError {
+    pub kind: CommandErrorKind,
+    pub error: Arc<dyn error::Error + Send + Sync>,
+    pub hints: Vec<ErrorHint>,
+}
+
+impl CommandError {
+    pub fn new(
+        kind: CommandErrorKind,
+        err: impl Into<Box<dyn error::Error + Send + Sync>>,
+    ) -> Self {
+        CommandError {
+            kind,
+            error: Arc::from(err.into()),
+            hints: vec![],
+        }
+    }
+
+    pub fn with_message(
+        kind: CommandErrorKind,
+        message: impl Into<String>,
+        source: impl Into<Box<dyn error::Error + Send + Sync>>,
+    ) -> Self {
+        Self::new(kind, ErrorWithMessage::new(message, source))
+    }
+
+    /// Returns error with the given plain-text `hint` attached.
+    pub fn hinted(mut self, hint: impl Into<String>) -> Self {
+        self.add_hint(hint);
+        self
+    }
+
+    /// Appends plain-text `hint` to the error.
+    pub fn add_hint(&mut self, hint: impl Into<String>) {
+        self.hints.push(ErrorHint::PlainText(hint.into()));
+    }
+
+    /// Appends formatted `hint` to the error.
+    pub fn add_formatted_hint(&mut self, hint: FormatRecorder) {
+        self.hints.push(ErrorHint::Formatted(hint));
+    }
+
+    /// Constructs formatted hint and appends it to the error.
+    pub fn add_formatted_hint_with(
+        &mut self,
+        write: impl FnOnce(&mut dyn Formatter) -> io::Result<()>,
+    ) {
+        let mut formatter = FormatRecorder::new();
+        write(&mut formatter).expect("write() to FormatRecorder should never fail");
+        self.add_formatted_hint(formatter);
+    }
+
+    /// Appends 0 or more plain-text `hints` to the error.
+    pub fn extend_hints(&mut self, hints: impl IntoIterator<Item = String>) {
+        self.hints
+            .extend(hints.into_iter().map(ErrorHint::PlainText));
+    }
+}
+
+#[derive(Clone, Debug)]
+pub enum ErrorHint {
+    PlainText(String),
+    Formatted(FormatRecorder),
 }
 
 /// Wraps error with user-visible message.
@@ -80,50 +140,47 @@ impl ErrorWithMessage {
 }
 
 pub fn user_error(err: impl Into<Box<dyn error::Error + Send + Sync>>) -> CommandError {
-    user_error_with_hint_opt(err, None)
+    CommandError::new(CommandErrorKind::User, err)
 }
 
 pub fn user_error_with_hint(
     err: impl Into<Box<dyn error::Error + Send + Sync>>,
     hint: impl Into<String>,
 ) -> CommandError {
-    user_error_with_hint_opt(err, Some(hint.into()))
+    user_error(err).hinted(hint)
 }
 
 pub fn user_error_with_message(
     message: impl Into<String>,
     source: impl Into<Box<dyn error::Error + Send + Sync>>,
 ) -> CommandError {
-    user_error_with_hint_opt(ErrorWithMessage::new(message, source), None)
+    CommandError::with_message(CommandErrorKind::User, message, source)
 }
 
-pub fn user_error_with_message_and_hint(
+pub fn config_error(err: impl Into<Box<dyn error::Error + Send + Sync>>) -> CommandError {
+    CommandError::new(CommandErrorKind::Config, err)
+}
+
+pub fn config_error_with_message(
     message: impl Into<String>,
-    hint: impl Into<String>,
     source: impl Into<Box<dyn error::Error + Send + Sync>>,
 ) -> CommandError {
-    user_error_with_hint_opt(ErrorWithMessage::new(message, source), Some(hint.into()))
+    CommandError::with_message(CommandErrorKind::Config, message, source)
 }
 
-pub fn user_error_with_hint_opt(
-    err: impl Into<Box<dyn error::Error + Send + Sync>>,
-    hint: Option<String>,
-) -> CommandError {
-    CommandError::UserError {
-        err: Arc::from(err.into()),
-        hint,
-    }
+pub fn cli_error(err: impl Into<Box<dyn error::Error + Send + Sync>>) -> CommandError {
+    CommandError::new(CommandErrorKind::Cli, err)
 }
 
 pub fn internal_error(err: impl Into<Box<dyn error::Error + Send + Sync>>) -> CommandError {
-    CommandError::InternalError(Arc::from(err.into()))
+    CommandError::new(CommandErrorKind::Internal, err)
 }
 
 pub fn internal_error_with_message(
     message: impl Into<String>,
     source: impl Into<Box<dyn error::Error + Send + Sync>>,
 ) -> CommandError {
-    CommandError::InternalError(Arc::new(ErrorWithMessage::new(message, source)))
+    CommandError::with_message(CommandErrorKind::Internal, message, source)
 }
 
 fn format_similarity_hint<S: AsRef<str>>(candidates: &[S]) -> Option<String> {
@@ -141,23 +198,23 @@ fn format_similarity_hint<S: AsRef<str>>(candidates: &[S]) -> Option<String> {
 
 impl From<io::Error> for CommandError {
     fn from(err: io::Error) -> Self {
-        if err.kind() == io::ErrorKind::BrokenPipe {
-            CommandError::BrokenPipe
-        } else {
-            user_error(err)
-        }
+        let kind = match err.kind() {
+            io::ErrorKind::BrokenPipe => CommandErrorKind::BrokenPipe,
+            _ => CommandErrorKind::User,
+        };
+        CommandError::new(kind, err)
     }
 }
 
 impl From<config::ConfigError> for CommandError {
     fn from(err: config::ConfigError) -> Self {
-        CommandError::ConfigError(err.to_string())
+        config_error(err)
     }
 }
 
 impl From<crate::config::ConfigError> for CommandError {
     fn from(err: crate::config::ConfigError) -> Self {
-        CommandError::ConfigError(err.to_string())
+        config_error(err)
     }
 }
 
@@ -181,7 +238,10 @@ impl From<CheckOutCommitError> for CommandError {
 
 impl From<BackendError> for CommandError {
     fn from(err: BackendError) -> Self {
-        internal_error_with_message("Unexpected error from backend", err)
+        match &err {
+            BackendError::Unsupported(_) => user_error(err),
+            _ => internal_error_with_message("Unexpected error from backend", err),
+        }
     }
 }
 
@@ -235,23 +295,17 @@ impl From<OpsetEvaluationError> for CommandError {
 impl From<SnapshotError> for CommandError {
     fn from(err: SnapshotError) -> Self {
         match err {
-            SnapshotError::NewFileTooLarge { .. } => user_error_with_message_and_hint(
-                "Failed to snapshot the working copy",
-                r#"Increase the value of the `snapshot.max-new-file-size` config option if you
+            SnapshotError::NewFileTooLarge { .. } => {
+                user_error_with_message("Failed to snapshot the working copy", err).hinted(
+                    r#"Increase the value of the `snapshot.max-new-file-size` config option if you
 want this file to be snapshotted. Otherwise add it to your `.gitignore` file."#,
-                err,
-            ),
+                )
+            }
             err => internal_error_with_message("Failed to snapshot the working copy", err),
         }
     }
 }
 
-impl From<TreeMergeError> for CommandError {
-    fn from(err: TreeMergeError) -> Self {
-        internal_error_with_message("Merge failed", err)
-    }
-}
-
 impl From<OpStoreError> for CommandError {
     fn from(err: OpStoreError) -> Self {
         internal_error_with_message("Failed to load an operation", err)
@@ -302,7 +356,6 @@ impl From<git2::Error> for CommandError {
 
 impl From<GitImportError> for CommandError {
     fn from(err: GitImportError) -> Self {
-        let message = format!("Failed to import refs from underlying Git repo: {err}");
         let hint = match &err {
             GitImportError::MissingHeadTarget { .. }
             | GitImportError::MissingRefAncestor { .. } => Some(
@@ -322,7 +375,10 @@ repository contents."
             GitImportError::InternalGitError(_) => None,
             GitImportError::UnexpectedBackend => None,
         };
-        user_error_with_hint_opt(message, hint)
+        let mut cmd_err =
+            user_error_with_message("Failed to import refs from underlying Git repo", err);
+        cmd_err.extend_hints(hint);
+        cmd_err
     }
 }
 
@@ -346,10 +402,9 @@ impl From<RevsetEvaluationError> for CommandError {
 
 impl From<RevsetParseError> for CommandError {
     fn from(err: RevsetParseError) -> Self {
-        let err_chain = iter::successors(Some(&err), |e| e.origin());
-        let message = err_chain.clone().join("\n");
         // Only for the bottom error, which is usually the root cause
-        let hint = match err_chain.last().unwrap().kind() {
+        let bottom_err = iter::successors(Some(&err), |e| e.origin()).last().unwrap();
+        let hint = match bottom_err.kind() {
             RevsetParseErrorKind::NotPrefixOperator {
                 op: _,
                 similar_op,
@@ -369,9 +424,15 @@ impl From<RevsetParseError> for CommandError {
                 name: _,
                 candidates,
             } => format_similarity_hint(candidates),
+            RevsetParseErrorKind::InvalidFunctionArguments { .. } => {
+                find_source_parse_error_hint(bottom_err)
+            }
             _ => None,
         };
-        user_error_with_hint_opt(format!("Failed to parse revset: {message}"), hint)
+        let mut cmd_err =
+            user_error_with_message(format!("Failed to parse revset: {}", err.kind()), err);
+        cmd_err.extend_hints(hint);
+        cmd_err
     }
 }
 
@@ -388,39 +449,56 @@ impl From<RevsetResolutionError> for CommandError {
             | RevsetResolutionError::AmbiguousChangeIdPrefix(_)
             | RevsetResolutionError::StoreError(_) => None,
         };
-        user_error_with_hint_opt(err, hint)
+        let mut cmd_err = user_error(err);
+        cmd_err.extend_hints(hint);
+        cmd_err
+    }
+}
+
+impl From<UserRevsetEvaluationError> for CommandError {
+    fn from(err: UserRevsetEvaluationError) -> Self {
+        match err {
+            UserRevsetEvaluationError::Resolution(err) => err.into(),
+            UserRevsetEvaluationError::Evaluation(err) => err.into(),
+        }
     }
 }
 
 impl From<TemplateParseError> for CommandError {
     fn from(err: TemplateParseError) -> Self {
-        let err_chain = iter::successors(Some(&err), |e| e.origin());
-        let message = err_chain.clone().join("\n");
         // Only for the bottom error, which is usually the root cause
-        let hint = match err_chain.last().unwrap().kind() {
+        let bottom_err = iter::successors(Some(&err), |e| e.origin()).last().unwrap();
+        let hint = match bottom_err.kind() {
             TemplateParseErrorKind::NoSuchKeyword { candidates, .. }
             | TemplateParseErrorKind::NoSuchFunction { candidates, .. }
             | TemplateParseErrorKind::NoSuchMethod { candidates, .. } => {
                 format_similarity_hint(candidates)
             }
+            TemplateParseErrorKind::InvalidArguments { .. }
+            | TemplateParseErrorKind::Expression(_) => find_source_parse_error_hint(bottom_err),
             _ => None,
         };
-        user_error_with_hint_opt(format!("Failed to parse template: {message}"), hint)
+        let mut cmd_err =
+            user_error_with_message(format!("Failed to parse template: {}", err.kind()), err);
+        cmd_err.extend_hints(hint);
+        cmd_err
     }
 }
 
 impl From<FsPathParseError> for CommandError {
     fn from(err: FsPathParseError) -> Self {
+        // TODO: implement pattern prefix like "root:<path>" or "--cwd" option,
+        // and suggest it if the user input looks like repo-relative path #3216.
         user_error(err)
     }
 }
 
 impl From<clap::Error> for CommandError {
     fn from(err: clap::Error) -> Self {
-        CommandError::ClapCliError {
-            err: Arc::new(err),
-            hint: None,
-        }
+        let hint = find_source_parse_error_hint(&err);
+        let mut cmd_err = cli_error(err);
+        cmd_err.extend_hints(hint);
+        cmd_err
     }
 }
 
@@ -442,6 +520,24 @@ impl From<GitIgnoreError> for CommandError {
     }
 }
 
+fn find_source_parse_error_hint(err: &dyn error::Error) -> Option<String> {
+    let source = err.source()?;
+    if let Some(source) = source.downcast_ref() {
+        string_pattern_parse_error_hint(source)
+    } else {
+        None
+    }
+}
+
+fn string_pattern_parse_error_hint(err: &StringPatternParseError) -> Option<String> {
+    match err {
+        StringPatternParseError::InvalidKind(_) => {
+            Some("Try prefixing with one of `exact:`, `glob:` or `substring:`".into())
+        }
+        StringPatternParseError::GlobPattern(_) => None,
+    }
+}
+
 const BROKEN_PIPE_EXIT_CODE: u8 = 3;
 
 pub(crate) fn handle_command_result(ui: &mut Ui, result: Result<(), CommandError>) -> ExitCode {
@@ -452,80 +548,121 @@ fn try_handle_command_result(
     ui: &mut Ui,
     result: Result<(), CommandError>,
 ) -> io::Result<ExitCode> {
-    match &result {
-        Ok(()) => Ok(ExitCode::SUCCESS),
-        Err(CommandError::UserError { err, hint }) => {
-            writeln!(ui.error(), "Error: {err}")?;
-            print_error_sources(ui, err.source())?;
-            if let Some(hint) = hint {
-                writeln!(ui.hint(), "Hint: {hint}")?;
-            }
+    let Err(cmd_err) = &result else {
+        return Ok(ExitCode::SUCCESS);
+    };
+    let err = &cmd_err.error;
+    let hints = &cmd_err.hints;
+    match cmd_err.kind {
+        CommandErrorKind::User => {
+            print_error(ui, "Error: ", err, hints)?;
             Ok(ExitCode::from(1))
         }
-        Err(CommandError::ConfigError(message)) => {
-            writeln!(ui.error(), "Config error: {message}")?;
+        CommandErrorKind::Config => {
+            print_error(ui, "Config error: ", err, hints)?;
             writeln!(
-                ui.hint(),
+                ui.stderr_formatter().labeled("hint"),
                 "For help, see https://github.com/martinvonz/jj/blob/main/docs/config.md."
             )?;
             Ok(ExitCode::from(1))
         }
-        Err(CommandError::CliError(message)) => {
-            writeln!(ui.error(), "Error: {message}")?;
-            Ok(ExitCode::from(2))
-        }
-        Err(CommandError::ClapCliError { err, hint }) => {
-            let clap_str = if ui.color() {
-                err.render().ansi().to_string()
+        CommandErrorKind::Cli => {
+            if let Some(err) = err.downcast_ref::<clap::Error>() {
+                handle_clap_error(ui, err, hints)
             } else {
-                err.render().to_string()
-            };
-
-            match err.kind() {
-                clap::error::ErrorKind::DisplayHelp
-                | clap::error::ErrorKind::DisplayHelpOnMissingArgumentOrSubcommand => {
-                    ui.request_pager()
-                }
-                _ => {}
-            };
-            // Definitions for exit codes and streams come from
-            // https://github.com/clap-rs/clap/blob/master/src/error/mod.rs
-            match err.kind() {
-                clap::error::ErrorKind::DisplayHelp | clap::error::ErrorKind::DisplayVersion => {
-                    write!(ui.stdout(), "{clap_str}")?;
-                    return Ok(ExitCode::SUCCESS);
-                }
-                _ => {}
+                print_error(ui, "Error: ", err, hints)?;
+                Ok(ExitCode::from(2))
             }
-            write!(ui.stderr(), "{clap_str}")?;
-            if let Some(hint) = hint {
-                writeln!(ui.hint(), "Hint: {hint}")?;
-            }
-            Ok(ExitCode::from(2))
         }
-        Err(CommandError::BrokenPipe) => {
+        CommandErrorKind::BrokenPipe => {
             // A broken pipe is not an error, but a signal to exit gracefully.
             Ok(ExitCode::from(BROKEN_PIPE_EXIT_CODE))
         }
-        Err(CommandError::InternalError(err)) => {
-            writeln!(ui.error(), "Internal error: {err}")?;
-            print_error_sources(ui, err.source())?;
+        CommandErrorKind::Internal => {
+            print_error(ui, "Internal error: ", err, hints)?;
             Ok(ExitCode::from(255))
         }
     }
 }
 
+fn print_error(
+    ui: &Ui,
+    heading: &str,
+    err: &dyn error::Error,
+    hints: &[ErrorHint],
+) -> io::Result<()> {
+    writeln!(ui.error_with_heading(heading), "{err}")?;
+    print_error_sources(ui, err.source())?;
+    print_error_hints(ui, hints)?;
+    Ok(())
+}
+
 fn print_error_sources(ui: &Ui, source: Option<&dyn error::Error>) -> io::Result<()> {
     let Some(err) = source else {
         return Ok(());
     };
-    if err.source().is_none() {
-        writeln!(ui.stderr(), "Caused by: {err}")?;
-    } else {
-        writeln!(ui.stderr(), "Caused by:")?;
-        for (i, err) in iter::successors(Some(err), |err| err.source()).enumerate() {
-            writeln!(ui.stderr(), "{n}: {err}", n = i + 1)?;
-        }
+    ui.stderr_formatter()
+        .with_label("error_source", |formatter| {
+            if err.source().is_none() {
+                write!(formatter.labeled("heading"), "Caused by: ")?;
+                writeln!(formatter, "{err}")?;
+            } else {
+                writeln!(formatter.labeled("heading"), "Caused by:")?;
+                for (i, err) in iter::successors(Some(err), |err| err.source()).enumerate() {
+                    write!(formatter.labeled("heading"), "{}: ", i + 1)?;
+                    writeln!(formatter, "{err}")?;
+                }
+            }
+            Ok(())
+        })
+}
+
+fn print_error_hints(ui: &Ui, hints: &[ErrorHint]) -> io::Result<()> {
+    for hint in hints {
+        ui.stderr_formatter().with_label("hint", |formatter| {
+            write!(formatter.labeled("heading"), "Hint: ")?;
+            match hint {
+                ErrorHint::PlainText(message) => {
+                    writeln!(formatter, "{message}")?;
+                    Ok(())
+                }
+                ErrorHint::Formatted(recorded) => {
+                    recorded.replay(formatter)?;
+                    // Formatted hint is usually multi-line text, and it's
+                    // convenient if trailing "\n" doesn't have to be omitted.
+                    if !recorded.data().ends_with(b"\n") {
+                        writeln!(formatter)?;
+                    }
+                    Ok(())
+                }
+            }
+        })?;
     }
     Ok(())
 }
+
+fn handle_clap_error(ui: &mut Ui, err: &clap::Error, hints: &[ErrorHint]) -> io::Result<ExitCode> {
+    let clap_str = if ui.color() {
+        err.render().ansi().to_string()
+    } else {
+        err.render().to_string()
+    };
+
+    match err.kind() {
+        clap::error::ErrorKind::DisplayHelp
+        | clap::error::ErrorKind::DisplayHelpOnMissingArgumentOrSubcommand => ui.request_pager(),
+        _ => {}
+    };
+    // Definitions for exit codes and streams come from
+    // https://github.com/clap-rs/clap/blob/master/src/error/mod.rs
+    match err.kind() {
+        clap::error::ErrorKind::DisplayHelp | clap::error::ErrorKind::DisplayVersion => {
+            write!(ui.stdout(), "{clap_str}")?;
+            return Ok(ExitCode::SUCCESS);
+        }
+        _ => {}
+    }
+    write!(ui.stderr(), "{clap_str}")?;
+    print_error_hints(ui, hints)?;
+    Ok(ExitCode::from(2))
+}
