@@ -18,7 +18,6 @@ use std::io::Write as _;
 
 use clap::builder::NonEmptyStringValueParser;
 use itertools::Itertools;
-use jj_lib::backend::CommitId;
 use jj_lib::git;
 use jj_lib::object_id::ObjectId;
 use jj_lib::op_store::{RefTarget, RemoteRef};
@@ -27,9 +26,7 @@ use jj_lib::revset::{self, RevsetExpression};
 use jj_lib::str_util::StringPattern;
 use jj_lib::view::View;
 
-use crate::cli_util::{
-    parse_string_pattern, CommandHelper, RemoteBranchName, RemoteBranchNamePattern, RevisionArg,
-};
+use crate::cli_util::{CommandHelper, RemoteBranchName, RemoteBranchNamePattern, RevisionArg};
 use crate::command_error::{user_error, user_error_with_hint, CommandError};
 use crate::formatter::Formatter;
 use crate::ui::Ui;
@@ -78,7 +75,7 @@ pub struct BranchDeleteArgs {
     /// By default, the specified name matches exactly. Use `glob:` prefix to
     /// select branches by wildcard pattern. For details, see
     /// https://github.com/martinvonz/jj/blob/main/docs/revsets.md#string-patterns.
-    #[arg(required_unless_present_any(&["glob"]), value_parser = parse_string_pattern)]
+    #[arg(required_unless_present_any(&["glob"]), value_parser = StringPattern::parse)]
     pub names: Vec<StringPattern>,
 
     /// Deprecated. Please prefix the pattern with `glob:` instead.
@@ -100,20 +97,24 @@ pub struct BranchDeleteArgs {
 pub struct BranchListArgs {
     /// Show all tracking and non-tracking remote branches including the ones
     /// whose targets are synchronized with the local branches.
-    #[arg(long, short, conflicts_with_all = ["names", "revisions", "tracked"])]
-    all: bool,
+    #[arg(long, short, alias = "all")]
+    all_remotes: bool,
 
     /// Show remote tracked branches only. Omits local Git-tracking branches by
     /// default.
-    #[arg(long, short, conflicts_with_all = ["all"])]
+    #[arg(long, short, conflicts_with_all = ["all_remotes"])]
     tracked: bool,
 
+    /// Show conflicted branches only.
+    #[arg(long, short, conflicts_with_all = ["all_remotes"])]
+    conflicted: bool,
+
     /// Show branches whose local name matches
     ///
     /// By default, the specified name matches exactly. Use `glob:` prefix to
     /// select branches by wildcard pattern. For details, see
     /// https://github.com/martinvonz/jj/blob/main/docs/revsets.md#string-patterns.
-    #[arg(value_parser = parse_string_pattern)]
+    #[arg(value_parser = StringPattern::parse)]
     pub names: Vec<StringPattern>,
 
     /// Show branches whose local targets are in the given revisions.
@@ -136,7 +137,7 @@ pub struct BranchForgetArgs {
     /// By default, the specified name matches exactly. Use `glob:` prefix to
     /// select branches by wildcard pattern. For details, see
     /// https://github.com/martinvonz/jj/blob/main/docs/revsets.md#string-patterns.
-    #[arg(required_unless_present_any(&["glob"]), value_parser = parse_string_pattern)]
+    #[arg(required_unless_present_any(&["glob"]), value_parser = StringPattern::parse)]
     pub names: Vec<StringPattern>,
 
     /// Deprecated. Please prefix the pattern with `glob:` instead.
@@ -239,7 +240,7 @@ fn cmd_branch_create(
 ) -> Result<(), CommandError> {
     let mut workspace_command = command.workspace_helper(ui)?;
     let target_commit =
-        workspace_command.resolve_single_rev(args.revision.as_deref().unwrap_or("@"))?;
+        workspace_command.resolve_single_rev(args.revision.as_ref().unwrap_or(&RevisionArg::AT))?;
     let view = workspace_command.repo().view();
     let branch_names = &args.names;
     if let Some(branch_name) = branch_names
@@ -254,8 +255,8 @@ fn cmd_branch_create(
 
     if branch_names.len() > 1 {
         writeln!(
-            ui.warning(),
-            "warning: Creating multiple branches: {}",
+            ui.warning_default(),
+            "Creating multiple branches: {}",
             branch_names.join(", "),
         )?;
     }
@@ -317,15 +318,17 @@ fn cmd_branch_rename(
         .any(|(_, remote_ref)| remote_ref.is_tracking())
     {
         writeln!(
-            ui.warning(),
-            "Warning: Branch {old_branch} has tracking remote branches which were not renamed."
-        )?;
-        writeln!(
-            ui.hint(),
-            "Hint: to rename the branch on the remote, you can `jj git push --branch \
-             {old_branch}` first (to delete it on the remote), and then `jj git push --branch \
-             {new_branch}`. `jj git push --all` would also be sufficient."
+            ui.warning_default(),
+            "Branch {old_branch} has tracking remote branches which were not renamed."
         )?;
+        if let Some(mut writer) = ui.hint_default() {
+            writeln!(
+                writer,
+                "to rename the branch on the remote, you can `jj git push --branch {old_branch}` \
+                 first (to delete it on the remote), and then `jj git push --branch \
+                 {new_branch}`. `jj git push --all` would also be sufficient."
+            )?;
+        }
     }
 
     Ok(())
@@ -338,7 +341,7 @@ fn cmd_branch_set(
 ) -> Result<(), CommandError> {
     let mut workspace_command = command.workspace_helper(ui)?;
     let target_commit =
-        workspace_command.resolve_single_rev(args.revision.as_deref().unwrap_or("@"))?;
+        workspace_command.resolve_single_rev(args.revision.as_ref().unwrap_or(&RevisionArg::AT))?;
     let repo = workspace_command.repo().as_ref();
     let is_fast_forward = |old_target: &RefTarget| {
         // Strictly speaking, "all" old targets should be ancestors, but we allow
@@ -366,8 +369,8 @@ fn cmd_branch_set(
 
     if branch_names.len() > 1 {
         writeln!(
-            ui.warning(),
-            "warning: Updating multiple branches: {}",
+            ui.warning_default(),
+            "Updating multiple branches: {}",
             branch_names.join(", "),
         )?;
     }
@@ -483,7 +486,7 @@ fn cmd_branch_delete(
     let view = workspace_command.repo().view();
     if !args.glob.is_empty() {
         writeln!(
-            ui.warning(),
+            ui.warning_default(),
             "--glob has been deprecated. Please prefix the pattern with `glob:` instead."
         )?;
     }
@@ -496,7 +499,7 @@ fn cmd_branch_delete(
     }
     tx.finish(ui, format!("delete {}", make_branch_term(&names)))?;
     if names.len() > 1 {
-        writeln!(ui.stderr(), "Deleted {} branches.", names.len())?;
+        writeln!(ui.status(), "Deleted {} branches.", names.len())?;
     }
     Ok(())
 }
@@ -510,7 +513,7 @@ fn cmd_branch_forget(
     let view = workspace_command.repo().view();
     if !args.glob.is_empty() {
         writeln!(
-            ui.warning(),
+            ui.warning_default(),
             "--glob has been deprecated. Please prefix the pattern with `glob:` instead."
         )?;
     }
@@ -522,7 +525,7 @@ fn cmd_branch_forget(
     }
     tx.finish(ui, format!("forget {}", make_branch_term(&names)))?;
     if names.len() > 1 {
-        writeln!(ui.stderr(), "Forgot {} branches.", names.len())?;
+        writeln!(ui.status(), "Forgot {} branches.", names.len())?;
     }
     Ok(())
 }
@@ -537,7 +540,10 @@ fn cmd_branch_track(
     let mut names = Vec::new();
     for (name, remote_ref) in find_remote_branches(view, &args.names)? {
         if remote_ref.is_tracking() {
-            writeln!(ui.warning(), "Remote branch already tracked: {name}")?;
+            writeln!(
+                ui.warning_default(),
+                "Remote branch already tracked: {name}"
+            )?;
         } else {
             names.push(name);
         }
@@ -550,7 +556,7 @@ fn cmd_branch_track(
     tx.finish(ui, format!("track remote {}", make_branch_term(&names)))?;
     if names.len() > 1 {
         writeln!(
-            ui.stderr(),
+            ui.status(),
             "Started tracking {} remote branches.",
             names.len()
         )?;
@@ -570,11 +576,14 @@ fn cmd_branch_untrack(
         if name.remote == git::REMOTE_NAME_FOR_LOCAL_GIT_REPO {
             // This restriction can be lifted if we want to support untracked @git branches.
             writeln!(
-                ui.warning(),
+                ui.warning_default(),
                 "Git-tracking branch cannot be untracked: {name}"
             )?;
         } else if !remote_ref.is_tracking() {
-            writeln!(ui.warning(), "Remote branch not tracked yet: {name}")?;
+            writeln!(
+                ui.warning_default(),
+                "Remote branch not tracked yet: {name}"
+            )?;
         } else {
             names.push(name);
         }
@@ -587,7 +596,7 @@ fn cmd_branch_untrack(
     tx.finish(ui, format!("untrack remote {}", make_branch_term(&names)))?;
     if names.len() > 1 {
         writeln!(
-            ui.stderr(),
+            ui.status(),
             "Stopped tracking {} remote branches.",
             names.len()
         )?;
@@ -616,18 +625,10 @@ fn cmd_branch_list(
         }
         if !args.revisions.is_empty() {
             // Match against local targets only, which is consistent with "jj git push".
-            let filter_expressions: Vec<_> = args
-                .revisions
-                .iter()
-                .map(|revision_str| workspace_command.parse_revset(revision_str))
-                .try_collect()?;
-            let filter_expression = RevsetExpression::union_all(&filter_expressions);
+            let mut expression = workspace_command.parse_union_revsets(&args.revisions)?;
             // Intersects with the set of local branch targets to minimize the lookup space.
-            let revset_expression = RevsetExpression::branches(StringPattern::everything())
-                .intersection(&filter_expression);
-            let revset_expression = revset::optimize(revset_expression);
-            let revset = workspace_command.evaluate_revset(revset_expression)?;
-            let filtered_targets: HashSet<CommitId> = revset.iter().collect();
+            expression.intersect_with(&RevsetExpression::branches(StringPattern::everything()));
+            let filtered_targets: HashSet<_> = expression.evaluate_to_commit_ids()?.collect();
             branch_names.extend(
                 view.local_branches()
                     .filter(|(_, target)| {
@@ -678,10 +679,11 @@ fn cmd_branch_list(
     let mut formatter = ui.stdout_formatter();
     let formatter = formatter.as_mut();
 
-    let branches_to_list = view.branches().filter(|&(name, _)| {
+    let branches_to_list = view.branches().filter(|(name, target)| {
         branch_names_to_list
             .as_ref()
             .map_or(true, |branch_names| branch_names.contains(name))
+            && (!args.conflicted || target.local_target.has_conflict())
     });
     for (name, branch_target) in branches_to_list {
         let (mut tracking_remote_refs, untracked_remote_refs) = branch_target
@@ -708,7 +710,7 @@ fn cmd_branch_list(
         for &(remote, remote_ref) in &tracking_remote_refs {
             let synced = remote_ref.target == *branch_target.local_target;
 
-            if !args.all && !args.tracked && synced {
+            if !args.all_remotes && !args.tracked && synced {
                 continue;
             }
             write!(formatter, "  ")?;
@@ -769,7 +771,7 @@ fn cmd_branch_list(
             }
         }
 
-        if args.all {
+        if args.all_remotes {
             for &(remote, remote_ref) in &untracked_remote_refs {
                 write!(formatter.labeled("branch"), "{name}@{remote}")?;
                 print_branch_target(formatter, &remote_ref.target)?;
