@@ -17,16 +17,20 @@ use std::fmt::Debug;
 use std::io::Write as _;
 
 use clap::Subcommand;
+use jj_lib::backend::TreeId;
 use jj_lib::default_index::{AsCompositeIndex as _, DefaultIndexStore, DefaultReadonlyIndex};
 use jj_lib::local_working_copy::LocalWorkingCopy;
+use jj_lib::merged_tree::MergedTree;
 use jj_lib::object_id::ObjectId;
+use jj_lib::repo::Repo;
+use jj_lib::repo_path::RepoPathBuf;
 use jj_lib::working_copy::WorkingCopy;
 use jj_lib::{op_walk, revset};
 
 use crate::cli_util::{CommandHelper, RevisionArg};
 use crate::command_error::{internal_error, user_error, CommandError};
-use crate::template_parser;
 use crate::ui::Ui;
+use crate::{revset_util, template_parser};
 
 /// Low-level commands not intended for users
 #[derive(Subcommand, Clone, Debug)]
@@ -94,8 +98,12 @@ pub enum DebugOperationDisplay {
 /// List the recursive entries of a tree.
 #[derive(clap::Args, Clone, Debug)]
 pub struct DebugTreeArgs {
-    #[arg(long, short = 'r', default_value = "@")]
-    revision: RevisionArg,
+    #[arg(long, short = 'r')]
+    revision: Option<RevisionArg>,
+    #[arg(long, conflicts_with = "revision")]
+    id: Option<String>,
+    #[arg(long, requires = "id")]
+    dir: Option<String>,
     paths: Vec<String>,
     // TODO: Add an option to include trees that are ancestors of the matched paths
 }
@@ -143,7 +151,8 @@ fn cmd_debug_revset(
     writeln!(ui.stdout(), "{expression:#?}")?;
     writeln!(ui.stdout())?;
 
-    let symbol_resolver = workspace_command.revset_symbol_resolver()?;
+    let symbol_resolver =
+        revset_util::default_symbol_resolver(repo, workspace_command.id_prefix_context()?);
     let expression = expression.resolve_user_expression(repo, &symbol_resolver)?;
     writeln!(ui.stdout(), "-- Resolved:")?;
     writeln!(ui.stdout(), "{expression:#?}")?;
@@ -250,7 +259,7 @@ fn cmd_debug_reindex(
             .build_index_at_operation(&op, repo_loader.store())
             .map_err(internal_error)?;
         writeln!(
-            ui.stderr(),
+            ui.status(),
             "Finished indexing {:?} commits.",
             default_index.as_composite().stats().num_commits
         )?;
@@ -292,8 +301,22 @@ fn cmd_debug_tree(
     args: &DebugTreeArgs,
 ) -> Result<(), CommandError> {
     let workspace_command = command.workspace_helper(ui)?;
-    let commit = workspace_command.resolve_single_rev(&args.revision)?;
-    let tree = commit.tree()?;
+    let tree = if let Some(tree_id_hex) = &args.id {
+        let tree_id =
+            TreeId::try_from_hex(tree_id_hex).map_err(|_| user_error("Invalid tree id"))?;
+        let dir = if let Some(dir_str) = &args.dir {
+            workspace_command.parse_file_path(dir_str)?
+        } else {
+            RepoPathBuf::root()
+        };
+        let store = workspace_command.repo().store();
+        let tree = store.get_tree(&dir, &tree_id)?;
+        MergedTree::resolved(tree)
+    } else {
+        let commit = workspace_command
+            .resolve_single_rev(args.revision.as_ref().unwrap_or(&RevisionArg::AT))?;
+        commit.tree()?
+    };
     let matcher = workspace_command.matcher_from_values(&args.paths)?;
     for (path, value) in tree.entries_matching(matcher.as_ref()) {
         let ui_path = workspace_command.format_file_path(&path);
@@ -335,7 +358,7 @@ fn cmd_debug_watchman(
             };
             locked_local_wc.reset_watchman()?;
             locked_ws.finish(repo.op_id().clone())?;
-            writeln!(ui.stderr(), "Reset Watchman clock")?;
+            writeln!(ui.status(), "Reset Watchman clock")?;
         }
     }
     Ok(())
