@@ -14,7 +14,6 @@
 
 use std::collections::HashSet;
 use std::io::Write;
-use std::ops::Deref;
 use std::path::{Path, PathBuf};
 use std::sync::Arc;
 use std::{fmt, fs, io};
@@ -41,17 +40,15 @@ use jj_lib::workspace::Workspace;
 use maplit::hashset;
 
 use crate::cli_util::{
-    parse_string_pattern, print_trackable_remote_branches, resolve_multiple_nonempty_revsets,
-    short_change_hash, short_commit_hash, start_repo_transaction, CommandHelper, RevisionArg,
-    WorkspaceCommandHelper,
+    print_trackable_remote_branches, short_change_hash, short_commit_hash, start_repo_transaction,
+    CommandHelper, RevisionArg, WorkspaceCommandHelper, WorkspaceCommandTransaction,
 };
 use crate::command_error::{
-    user_error, user_error_with_hint, user_error_with_hint_opt, user_error_with_message,
-    CommandError,
+    user_error, user_error_with_hint, user_error_with_message, CommandError,
 };
 use crate::git_util::{
     get_git_repo, is_colocated_git_workspace, print_failed_git_export, print_git_import_stats,
-    with_remote_git_callbacks,
+    with_remote_git_callbacks, GitSidebandProgressMessageWriter,
 };
 use crate::ui::Ui;
 
@@ -159,7 +156,7 @@ pub struct GitFetchArgs {
     ///
     /// By default, the specified name matches exactly. Use `glob:` prefix to
     /// expand `*` as a glob. The other wildcard characters aren't supported.
-    #[arg(long, short, default_value = "glob:*", value_parser = parse_string_pattern)]
+    #[arg(long, short, default_value = "glob:*", value_parser = StringPattern::parse)]
     branch: Vec<StringPattern>,
     /// The remote to fetch from (only named remotes are supported, can be
     /// repeated)
@@ -204,7 +201,7 @@ pub struct GitPushArgs {
     /// By default, the specified name matches exactly. Use `glob:` prefix to
     /// select branches by wildcard pattern. For details, see
     /// https://martinvonz.github.io/jj/latest/revsets#string-patterns.
-    #[arg(long, short, value_parser = parse_string_pattern)]
+    #[arg(long, short, value_parser = StringPattern::parse)]
     branch: Vec<StringPattern>,
     /// Push all branches (including deleted branches)
     #[arg(long)]
@@ -484,7 +481,7 @@ fn init_git_refs(
     }
     let repo = tx.commit("import git refs");
     writeln!(
-        ui.stderr(),
+        ui.status(),
         "Done importing changes from the underlying Git repo."
     )?;
     Ok(repo)
@@ -511,7 +508,7 @@ fn cmd_git_init(
 
     let relative_wc_path = file_util::relative_path(cwd, &wc_path);
     writeln!(
-        ui.stderr(),
+        ui.status(),
         r#"Initialized repo in "{}""#,
         relative_wc_path.display()
     )?;
@@ -536,7 +533,7 @@ fn cmd_git_fetch(
     };
     let mut tx = workspace_command.start_transaction();
     for remote in &remotes {
-        let stats = with_remote_git_callbacks(ui, |cb| {
+        let stats = with_remote_git_callbacks(ui, None, |cb| {
             git::fetch(
                 tx.mut_repo(),
                 &git_repo,
@@ -597,11 +594,9 @@ fn get_default_fetch_remotes(
     } else if let Some(remote) = get_single_remote(git_repo)? {
         // if nothing was explicitly configured, try to guess
         if remote != DEFAULT_REMOTE {
-            writeln!(
-                ui.hint(),
-                "Fetching from the only existing remote: {}",
-                remote
-            )?;
+            if let Some(mut writer) = ui.hint_default() {
+                writeln!(writer, "Fetching from the only existing remote: {remote}")?;
+            }
         }
         Ok(vec![remote])
     } else {
@@ -705,7 +700,7 @@ fn cmd_git_clone(
         };
         if let Err(err) = clean_up_dirs() {
             writeln!(
-                ui.warning(),
+                ui.warning_default(),
                 "Failed to clean up {}: {}",
                 canonical_wc_path.display(),
                 err
@@ -750,7 +745,7 @@ fn do_git_clone(
     };
     let git_repo = get_git_repo(repo.store())?;
     writeln!(
-        ui.stderr(),
+        ui.status(),
         r#"Fetching into new repo in "{}""#,
         wc_path.display()
     )?;
@@ -759,7 +754,7 @@ fn do_git_clone(
     git_repo.remote(remote_name, source).unwrap();
     let mut fetch_tx = workspace_command.start_transaction();
 
-    let stats = with_remote_git_callbacks(ui, |cb| {
+    let stats = with_remote_git_callbacks(ui, None, |cb| {
         git::fetch(
             fetch_tx.mut_repo(),
             &git_repo,
@@ -799,13 +794,6 @@ fn cmd_git_push(
     };
 
     let repo = workspace_command.repo().clone();
-    let wc_commit_id = workspace_command.get_wc_commit_id().cloned();
-    let change_commits: Vec<_> = args
-        .change
-        .iter()
-        .map(|change_str| workspace_command.resolve_single_rev(change_str))
-        .try_collect()?;
-
     let mut tx = workspace_command.start_transaction();
     let tx_description;
     let mut branch_updates = vec![];
@@ -843,65 +831,31 @@ fn cmd_git_push(
         }
         tx_description = format!("push all deleted branches to git remote {remote}");
     } else {
-        let mut seen_branches = hashset! {};
-        let branches_by_name =
-            find_branches_to_push(repo.view(), &args.branch, &remote, &mut seen_branches)?;
-        for (branch_name, targets) in branches_by_name {
+        let mut seen_branches: HashSet<&str> = HashSet::new();
+
+        // Process --change branches first because matching branches can be moved.
+        let change_branch_names = update_change_branches(
+            ui,
+            &mut tx,
+            &args.change,
+            &command.settings().push_branch_prefix(),
+        )?;
+        let change_branches = change_branch_names.iter().map(|branch_name| {
+            let targets = LocalAndRemoteRef {
+                local_target: tx.repo().view().get_local_branch(branch_name),
+                remote_ref: tx.repo().view().get_remote_branch(branch_name, &remote),
+            };
+            (branch_name.as_ref(), targets)
+        });
+        let branches_by_name = find_branches_to_push(repo.view(), &args.branch, &remote)?;
+        for (branch_name, targets) in change_branches.chain(branches_by_name.iter().copied()) {
+            if !seen_branches.insert(branch_name) {
+                continue;
+            }
             match classify_branch_update(branch_name, &remote, targets) {
                 Ok(Some(update)) => branch_updates.push((branch_name.to_owned(), update)),
                 Ok(None) => writeln!(
-                    ui.stderr(),
-                    "Branch {branch_name}@{remote} already matches {branch_name}",
-                )?,
-                Err(reason) => return Err(reason.into()),
-            }
-        }
-
-        for (change_str, commit) in std::iter::zip(args.change.iter(), change_commits) {
-            let mut branch_name = format!(
-                "{}{}",
-                command.settings().push_branch_prefix(),
-                commit.change_id().hex()
-            );
-            if !seen_branches.insert(branch_name.clone()) {
-                continue;
-            }
-            let view = tx.base_repo().view();
-            if view.get_local_branch(&branch_name).is_absent() {
-                // A local branch with the full change ID doesn't exist already, so use the
-                // short ID if it's not ambiguous (which it shouldn't be most of the time).
-                let short_change_id = short_change_hash(commit.change_id());
-                if tx
-                    .base_workspace_helper()
-                    .resolve_single_rev(&short_change_id)
-                    .is_ok()
-                {
-                    // Short change ID is not ambiguous, so update the branch name to use it.
-                    branch_name = format!(
-                        "{}{}",
-                        command.settings().push_branch_prefix(),
-                        short_change_id
-                    );
-                };
-            }
-            if view.get_local_branch(&branch_name).is_absent() {
-                writeln!(
-                    ui.stderr(),
-                    "Creating branch {} for revision {}",
-                    branch_name,
-                    change_str.deref()
-                )?;
-            }
-            tx.mut_repo()
-                .set_local_branch_target(&branch_name, RefTarget::normal(commit.id().clone()));
-            let targets = LocalAndRemoteRef {
-                local_target: tx.repo().view().get_local_branch(&branch_name),
-                remote_ref: tx.repo().view().get_remote_branch(&branch_name, &remote),
-            };
-            match classify_branch_update(&branch_name, &remote, targets) {
-                Ok(Some(update)) => branch_updates.push((branch_name.clone(), update)),
-                Ok(None) => writeln!(
-                    ui.stderr(),
+                    ui.status(),
                     "Branch {branch_name}@{remote} already matches {branch_name}",
                 )?,
                 Err(reason) => return Err(reason.into()),
@@ -910,38 +864,15 @@ fn cmd_git_push(
 
         let use_default_revset =
             args.branch.is_empty() && args.change.is_empty() && args.revisions.is_empty();
-        let revision_commit_ids: HashSet<_> = if use_default_revset {
-            let Some(wc_commit_id) = wc_commit_id else {
-                return Err(user_error("Nothing checked out in this workspace"));
-            };
-            let current_branches_expression = RevsetExpression::remote_branches(
-                StringPattern::everything(),
-                StringPattern::Exact(remote.to_owned()),
-            )
-            .range(&RevsetExpression::commit(wc_commit_id))
-            .intersection(&RevsetExpression::branches(StringPattern::everything()));
-            let current_branches_revset = tx
-                .base_workspace_helper()
-                .evaluate_revset(current_branches_expression)?;
-            current_branches_revset.iter().collect()
-        } else {
-            // TODO: Narrow search space to local target commits.
-            // TODO: Remove redundant CommitId -> Commit -> CommitId round trip.
-            resolve_multiple_nonempty_revsets(&args.revisions, tx.base_workspace_helper())?
-                .iter()
-                .map(|commit| commit.id().clone())
-                .collect()
-        };
-        let branches_targeted = repo
-            .view()
-            .local_remote_branches(&remote)
-            .filter(|(_, targets)| {
-                let mut local_ids = targets.local_target.added_ids();
-                local_ids.any(|id| revision_commit_ids.contains(id))
-            })
-            .collect_vec();
+        let branches_targeted = find_branches_targeted_by_revisions(
+            ui,
+            tx.base_workspace_helper(),
+            &remote,
+            &args.revisions,
+            use_default_revset,
+        )?;
         for &(branch_name, targets) in &branches_targeted {
-            if !seen_branches.insert(branch_name.to_owned()) {
+            if !seen_branches.insert(branch_name) {
                 continue;
             }
             match classify_branch_update(branch_name, &remote, targets) {
@@ -950,21 +881,6 @@ fn cmd_git_push(
                 Err(reason) => reason.print(ui)?,
             }
         }
-        if branches_targeted.is_empty() {
-            if use_default_revset {
-                writeln!(
-                    ui.warning(),
-                    "No branches found in the default push revset, \
-                     `remote_branches(remote={remote})..@`."
-                )?;
-            } else if !args.revisions.is_empty() {
-                writeln!(
-                    ui.warning(),
-                    "No branches point to the specified revisions."
-                )?;
-            } else { /* A plain "Nothing changed" message will suffice */
-            }
-        }
 
         tx_description = format!(
             "push {} to git remote {}",
@@ -978,7 +894,7 @@ fn cmd_git_push(
         );
     }
     if branch_updates.is_empty() {
-        writeln!(ui.stderr(), "Nothing changed.")?;
+        writeln!(ui.status(), "Nothing changed.")?;
         return Ok(());
     }
 
@@ -1040,20 +956,20 @@ fn cmd_git_push(
         }
     }
 
-    writeln!(ui.stderr(), "Branch changes to push to {}:", &remote)?;
+    writeln!(ui.status(), "Branch changes to push to {}:", &remote)?;
     for (branch_name, update) in &branch_updates {
         match (&update.old_target, &update.new_target) {
             (Some(old_target), Some(new_target)) => {
                 if force_pushed_branches.contains(branch_name) {
                     writeln!(
-                        ui.stderr(),
+                        ui.status(),
                         "  Force branch {branch_name} from {} to {}",
                         short_commit_hash(old_target),
                         short_commit_hash(new_target)
                     )?;
                 } else {
                     writeln!(
-                        ui.stderr(),
+                        ui.status(),
                         "  Move branch {branch_name} from {} to {}",
                         short_commit_hash(old_target),
                         short_commit_hash(new_target)
@@ -1062,14 +978,14 @@ fn cmd_git_push(
             }
             (Some(old_target), None) => {
                 writeln!(
-                    ui.stderr(),
+                    ui.status(),
                     "  Delete branch {branch_name} from {}",
                     short_commit_hash(old_target)
                 )?;
             }
             (None, Some(new_target)) => {
                 writeln!(
-                    ui.stderr(),
+                    ui.status(),
                     "  Add branch {branch_name} to {}",
                     short_commit_hash(new_target)
                 )?;
@@ -1081,7 +997,7 @@ fn cmd_git_push(
     }
 
     if args.dry_run {
-        writeln!(ui.stderr(), "Dry-run requested, not pushing.")?;
+        writeln!(ui.status(), "Dry-run requested, not pushing.")?;
         return Ok(());
     }
 
@@ -1089,7 +1005,11 @@ fn cmd_git_push(
         branch_updates,
         force_pushed_branches,
     };
-    with_remote_git_callbacks(ui, |cb| {
+    let mut writer = GitSidebandProgressMessageWriter::new(ui);
+    let mut sideband_progress_callback = |progress_message: &[u8]| {
+        _ = writer.write(ui, progress_message);
+    };
+    with_remote_git_callbacks(ui, Some(&mut sideband_progress_callback), |cb| {
         git::push_branches(tx.mut_repo(), &git_repo, &remote, &targets, cb)
     })
     .map_err(|err| match err {
@@ -1101,6 +1021,7 @@ fn cmd_git_push(
         ),
         _ => user_error(err),
     })?;
+    writer.flush(ui)?;
     tx.finish(ui, tx_description)?;
     Ok(())
 }
@@ -1115,7 +1036,9 @@ fn get_default_push_remote(
     } else if let Some(remote) = get_single_remote(git_repo)? {
         // similar to get_default_fetch_remotes
         if remote != DEFAULT_REMOTE {
-            writeln!(ui.hint(), "Pushing to the only existing remote: {}", remote)?;
+            if let Some(mut writer) = ui.hint_default() {
+                writeln!(writer, "Pushing to the only existing remote: {remote}")?;
+            }
         }
         Ok(remote)
     } else {
@@ -1131,9 +1054,11 @@ struct RejectedBranchUpdateReason {
 
 impl RejectedBranchUpdateReason {
     fn print(&self, ui: &Ui) -> io::Result<()> {
-        writeln!(ui.warning(), "{}", self.message)?;
+        writeln!(ui.warning_default(), "{}", self.message)?;
         if let Some(hint) = &self.hint {
-            writeln!(ui.hint(), "Hint: {hint}")?;
+            if let Some(mut writer) = ui.hint_default() {
+                writeln!(writer, "{hint}")?;
+            }
         }
         Ok(())
     }
@@ -1142,7 +1067,9 @@ impl RejectedBranchUpdateReason {
 impl From<RejectedBranchUpdateReason> for CommandError {
     fn from(reason: RejectedBranchUpdateReason) -> Self {
         let RejectedBranchUpdateReason { message, hint } = reason;
-        user_error_with_hint_opt(message, hint)
+        let mut cmd_err = user_error(message);
+        cmd_err.extend_hints(hint);
+        cmd_err
     }
 }
 
@@ -1174,11 +1101,48 @@ fn classify_branch_update(
     }
 }
 
+/// Creates or moves branches based on the change IDs.
+fn update_change_branches(
+    ui: &Ui,
+    tx: &mut WorkspaceCommandTransaction,
+    changes: &[RevisionArg],
+    branch_prefix: &str,
+) -> Result<Vec<String>, CommandError> {
+    let mut branch_names = Vec::new();
+    for change_arg in changes {
+        let workspace_command = tx.base_workspace_helper();
+        let commit = workspace_command.resolve_single_rev(change_arg)?;
+        let mut branch_name = format!("{branch_prefix}{}", commit.change_id().hex());
+        let view = tx.base_repo().view();
+        if view.get_local_branch(&branch_name).is_absent() {
+            // A local branch with the full change ID doesn't exist already, so use the
+            // short ID if it's not ambiguous (which it shouldn't be most of the time).
+            let short_change_id = short_change_hash(commit.change_id());
+            if workspace_command
+                .resolve_single_rev(&RevisionArg::from(short_change_id.clone()))
+                .is_ok()
+            {
+                // Short change ID is not ambiguous, so update the branch name to use it.
+                branch_name = format!("{branch_prefix}{short_change_id}");
+            };
+        }
+        if view.get_local_branch(&branch_name).is_absent() {
+            writeln!(
+                ui.status(),
+                "Creating branch {branch_name} for revision {change_arg}",
+            )?;
+        }
+        tx.mut_repo()
+            .set_local_branch_target(&branch_name, RefTarget::normal(commit.id().clone()));
+        branch_names.push(branch_name);
+    }
+    Ok(branch_names)
+}
+
 fn find_branches_to_push<'a>(
     view: &'a View,
     branch_patterns: &[StringPattern],
     remote_name: &str,
-    seen_branches: &mut HashSet<String>,
 ) -> Result<Vec<(&'a str, LocalAndRemoteRef<'a>)>, CommandError> {
     let mut matching_branches = vec![];
     let mut unmatched_patterns = vec![];
@@ -1194,8 +1158,7 @@ fn find_branches_to_push<'a>(
         if matches.peek().is_none() {
             unmatched_patterns.push(pattern);
         }
-        matching_branches
-            .extend(matches.filter(|&(name, _)| seen_branches.insert(name.to_owned())));
+        matching_branches.extend(matches);
     }
     match &unmatched_patterns[..] {
         [] => Ok(matching_branches),
@@ -1207,6 +1170,59 @@ fn find_branches_to_push<'a>(
     }
 }
 
+fn find_branches_targeted_by_revisions<'a>(
+    ui: &Ui,
+    workspace_command: &'a WorkspaceCommandHelper,
+    remote_name: &str,
+    revisions: &[RevisionArg],
+    use_default_revset: bool,
+) -> Result<Vec<(&'a str, LocalAndRemoteRef<'a>)>, CommandError> {
+    let mut revision_commit_ids = HashSet::new();
+    if use_default_revset {
+        let Some(wc_commit_id) = workspace_command.get_wc_commit_id().cloned() else {
+            return Err(user_error("Nothing checked out in this workspace"));
+        };
+        let current_branches_expression = RevsetExpression::remote_branches(
+            StringPattern::everything(),
+            StringPattern::Exact(remote_name.to_owned()),
+        )
+        .range(&RevsetExpression::commit(wc_commit_id))
+        .intersection(&RevsetExpression::branches(StringPattern::everything()));
+        let current_branches_revset =
+            current_branches_expression.evaluate_programmatic(workspace_command.repo().as_ref())?;
+        revision_commit_ids.extend(current_branches_revset.iter());
+        if revision_commit_ids.is_empty() {
+            writeln!(
+                ui.warning_default(),
+                "No branches found in the default push revset: \
+                 remote_branches(remote={remote_name})..@"
+            )?;
+        }
+    }
+    for rev_arg in revisions {
+        let mut expression = workspace_command.parse_revset(rev_arg)?;
+        expression.intersect_with(&RevsetExpression::branches(StringPattern::everything()));
+        let mut commit_ids = expression.evaluate_to_commit_ids()?.peekable();
+        if commit_ids.peek().is_none() {
+            writeln!(
+                ui.warning_default(),
+                "No branches point to the specified revisions: {rev_arg}"
+            )?;
+        }
+        revision_commit_ids.extend(commit_ids);
+    }
+    let branches_targeted = workspace_command
+        .repo()
+        .view()
+        .local_remote_branches(remote_name)
+        .filter(|(_, targets)| {
+            let mut local_ids = targets.local_target.added_ids();
+            local_ids.any(|id| revision_commit_ids.contains(id))
+        })
+        .collect_vec();
+    Ok(branches_targeted)
+}
+
 fn cmd_git_import(
     ui: &mut Ui,
     command: &CommandHelper,
@@ -1248,7 +1264,7 @@ fn cmd_git_submodule_print_gitmodules(
     let gitmodules_path = RepoPath::from_internal_string(".gitmodules");
     let mut gitmodules_file = match tree.path_value(gitmodules_path).into_resolved() {
         Ok(None) => {
-            writeln!(ui.stderr(), "No submodules!")?;
+            writeln!(ui.status(), "No submodules!")?;
             return Ok(());
         }
         Ok(Some(TreeValue::File { id, .. })) => repo.store().read_file(gitmodules_path, &id)?,
