@@ -12,6 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+use std::borrow::Borrow;
 use std::collections::HashMap;
 use std::io::Write;
 use std::sync::Arc;
@@ -29,10 +30,11 @@ use jj_lib::settings::UserSettings;
 use tracing::instrument;
 
 use crate::cli_util::{
-    self, resolve_multiple_nonempty_revsets_default_single, short_commit_hash, CommandHelper,
-    RevisionArg, WorkspaceCommandHelper,
+    short_commit_hash, CommandHelper, RevisionArg, WorkspaceCommandHelper,
+    WorkspaceCommandTransaction,
 };
 use crate::command_error::{user_error, CommandError};
+use crate::formatter::Formatter;
 use crate::ui::Ui;
 
 /// Move revisions to different parent(s)
@@ -134,8 +136,8 @@ pub(crate) struct RebaseArgs {
     #[arg(long, short)]
     branch: Vec<RevisionArg>,
 
-    /// Rebase specified revision(s) together their tree of descendants (can be
-    /// repeated)
+    /// Rebase specified revision(s) together with their trees of descendants
+    /// (can be repeated)
     ///
     /// Each specified revision will become a direct child of the destination
     /// revision(s), even if some of the source revisions are descendants
@@ -191,10 +193,11 @@ Please use `jj rebase -d 'all:x|y'` instead of `jj rebase --allow-large-revsets
         simplify_ancestor_merge: false,
     };
     let mut workspace_command = command.workspace_helper(ui)?;
-    let new_parents = cli_util::resolve_all_revs(&workspace_command, &args.destination)?
+    let new_parents = workspace_command
+        .resolve_some_revsets_default_single(&args.destination)?
         .into_iter()
         .collect_vec();
-    if let Some(rev_str) = &args.revision {
+    if let Some(rev_arg) = &args.revision {
         assert_eq!(
             // In principle, `-r --skip-empty` could mean to abandon the `-r`
             // commit if it becomes empty. This seems internally consistent with
@@ -215,12 +218,11 @@ Please use `jj rebase -d 'all:x|y'` instead of `jj rebase --allow-large-revsets
             command.settings(),
             &mut workspace_command,
             &new_parents,
-            rev_str,
+            rev_arg,
         )?;
     } else if !args.source.is_empty() {
-        let source_commits =
-            resolve_multiple_nonempty_revsets_default_single(&workspace_command, &args.source)?;
-        rebase_descendants(
+        let source_commits = workspace_command.resolve_some_revsets_default_single(&args.source)?;
+        rebase_descendants_transaction(
             ui,
             command.settings(),
             &mut workspace_command,
@@ -230,9 +232,9 @@ Please use `jj rebase -d 'all:x|y'` instead of `jj rebase --allow-large-revsets
         )?;
     } else {
         let branch_commits = if args.branch.is_empty() {
-            IndexSet::from([workspace_command.resolve_single_rev("@")?])
+            IndexSet::from([workspace_command.resolve_single_rev(&RevisionArg::AT)?])
         } else {
-            resolve_multiple_nonempty_revsets_default_single(&workspace_command, &args.branch)?
+            workspace_command.resolve_some_revsets_default_single(&args.branch)?
         };
         rebase_branch(
             ui,
@@ -271,7 +273,7 @@ fn rebase_branch(
         .iter()
         .commits(workspace_command.repo().store())
         .try_collect()?;
-    rebase_descendants(
+    rebase_descendants_transaction(
         ui,
         settings,
         workspace_command,
@@ -281,7 +283,30 @@ fn rebase_branch(
     )
 }
 
-fn rebase_descendants(
+/// Rebases `old_commits` onto `new_parents`.
+pub fn rebase_descendants(
+    tx: &mut WorkspaceCommandTransaction,
+    settings: &UserSettings,
+    new_parents: &[Commit],
+    old_commits: &[impl Borrow<Commit>],
+    rebase_options: RebaseOptions,
+) -> Result<usize, CommandError> {
+    for old_commit in old_commits.iter() {
+        rebase_commit_with_options(
+            settings,
+            tx.mut_repo(),
+            old_commit.borrow(),
+            new_parents,
+            &rebase_options,
+        )?;
+    }
+    let num_rebased = old_commits.len()
+        + tx.mut_repo()
+            .rebase_descendants_with_options(settings, rebase_options)?;
+    Ok(num_rebased)
+}
+
+fn rebase_descendants_transaction(
     ui: &mut Ui,
     settings: &UserSettings,
     workspace_command: &mut WorkspaceCommandHelper,
@@ -290,25 +315,24 @@ fn rebase_descendants(
     rebase_options: RebaseOptions,
 ) -> Result<(), CommandError> {
     workspace_command.check_rewritable(old_commits)?;
+    let (skipped_commits, old_commits) = old_commits
+        .iter()
+        .partition::<Vec<_>, _>(|commit| commit.parents() == new_parents);
+    if !skipped_commits.is_empty() {
+        if let Some(mut fmt) = ui.status_formatter() {
+            log_skipped_rebase_commits_message(fmt.as_mut(), workspace_command, &skipped_commits)?;
+        }
+    }
+    if old_commits.is_empty() {
+        return Ok(());
+    }
     for old_commit in old_commits.iter() {
         check_rebase_destinations(workspace_command.repo(), new_parents, old_commit)?;
     }
     let mut tx = workspace_command.start_transaction();
-    // `rebase_descendants` takes care of sorting in reverse topological order, so
-    // no need to do it here.
-    for old_commit in old_commits {
-        rebase_commit_with_options(
-            settings,
-            tx.mut_repo(),
-            old_commit,
-            new_parents,
-            &rebase_options,
-        )?;
-    }
-    let num_rebased = old_commits.len()
-        + tx.mut_repo()
-            .rebase_descendants_with_options(settings, rebase_options)?;
-    writeln!(ui.stderr(), "Rebased {num_rebased} commits")?;
+    let num_rebased =
+        rebase_descendants(&mut tx, settings, new_parents, &old_commits, rebase_options)?;
+    writeln!(ui.status(), "Rebased {num_rebased} commits")?;
     let tx_message = if old_commits.len() == 1 {
         format!(
             "rebase commit {} and descendants",
@@ -326,9 +350,9 @@ fn rebase_revision(
     settings: &UserSettings,
     workspace_command: &mut WorkspaceCommandHelper,
     new_parents: &[Commit],
-    rev_str: &str,
+    rev_arg: &RevisionArg,
 ) -> Result<(), CommandError> {
-    let old_commit = workspace_command.resolve_single_rev(rev_str)?;
+    let old_commit = workspace_command.resolve_single_rev(rev_arg)?;
     workspace_command.check_rewritable([&old_commit])?;
     if new_parents.contains(&old_commit) {
         return Err(user_error(format!(
@@ -344,7 +368,7 @@ fn rebase_revision(
         .iter()
         .commits(workspace_command.repo().store())
         .try_collect()?;
-    // Currently, immutable commits are defied so that a child of a rewriteable
+    // Currently, immutable commits are defined so that a child of a rewriteable
     // commit is always rewriteable.
     debug_assert!(workspace_command.check_rewritable(&child_commits).is_ok());
 
@@ -432,21 +456,42 @@ fn rebase_revision(
         })
         .try_collect()?;
 
-    // Finally, it's safe to rebase `old_commit`. At this point, it should no longer
-    // have any children; they have all been rebased and the originals have been
-    // abandoned.
-    rebase_commit(settings, tx.mut_repo(), &old_commit, &new_parents)?;
-    debug_assert_eq!(tx.mut_repo().rebase_descendants(settings)?, 0);
+    // Finally, it's safe to rebase `old_commit`. We can skip rebasing if it is
+    // already a child of `new_parents`. Otherwise, at this point, it should no
+    // longer have any children; they have all been rebased and the originals
+    // have been abandoned.
+    let skipped_commit_rebase = if old_commit.parents() == new_parents {
+        if let Some(mut formatter) = ui.status_formatter() {
+            write!(formatter, "Skipping rebase of commit ")?;
+            tx.write_commit_summary(formatter.as_mut(), &old_commit)?;
+            writeln!(formatter)?;
+        }
+        true
+    } else {
+        rebase_commit(settings, tx.mut_repo(), &old_commit, &new_parents)?;
+        debug_assert_eq!(tx.mut_repo().rebase_descendants(settings)?, 0);
+        false
+    };
 
     if num_rebased_descendants > 0 {
-        writeln!(
-            ui.stderr(),
-            "Also rebased {num_rebased_descendants} descendant commits onto parent of rebased \
-             commit"
-        )?;
+        if skipped_commit_rebase {
+            writeln!(
+                ui.status(),
+                "Rebased {num_rebased_descendants} descendant commits onto parent of commit"
+            )?;
+        } else {
+            writeln!(
+                ui.status(),
+                "Also rebased {num_rebased_descendants} descendant commits onto parent of rebased \
+                 commit"
+            )?;
+        }
+    }
+    if tx.mut_repo().has_changes() {
+        tx.finish(ui, format!("rebase commit {}", old_commit.id().hex()))
+    } else {
+        Ok(()) // Do not print "Nothing changed."
     }
-    tx.finish(ui, format!("rebase commit {}", old_commit.id().hex()))?;
-    Ok(())
 }
 
 fn check_rebase_destinations(
@@ -465,3 +510,24 @@ fn check_rebase_destinations(
     }
     Ok(())
 }
+
+fn log_skipped_rebase_commits_message(
+    fmt: &mut dyn Formatter,
+    workspace_command: &WorkspaceCommandHelper,
+    commits: &[&Commit],
+) -> Result<(), CommandError> {
+    let template = workspace_command.commit_summary_template();
+    if commits.len() == 1 {
+        write!(fmt, "Skipping rebase of commit ")?;
+        template.format(commits[0], fmt)?;
+        writeln!(fmt)?;
+    } else {
+        writeln!(fmt, "Skipping rebase of commits:")?;
+        for commit in commits {
+            write!(fmt, "  ")?;
+            template.format(commit, fmt)?;
+            writeln!(fmt)?;
+        }
+    }
+    Ok(())
+}
