@@ -12,6 +12,7 @@
 // See the License for the specific language governing permissions and
 // limitations under the License.
 
+use std::any::Any;
 use std::cmp::max;
 use std::collections::HashMap;
 use std::io;
@@ -20,36 +21,46 @@ use std::rc::Rc;
 use itertools::Itertools as _;
 use jj_lib::backend::{ChangeId, CommitId};
 use jj_lib::commit::Commit;
+use jj_lib::extensions_map::ExtensionsMap;
 use jj_lib::hex_util::to_reverse_hex;
 use jj_lib::id_prefix::IdPrefixContext;
 use jj_lib::object_id::ObjectId as _;
 use jj_lib::op_store::{RefTarget, WorkspaceId};
 use jj_lib::repo::Repo;
+use jj_lib::revset::{Revset, RevsetParseContext};
 use jj_lib::{git, rewrite};
 use once_cell::unsync::OnceCell;
 
-use crate::formatter::Formatter;
 use crate::template_builder::{
     self, merge_fn_map, BuildContext, CoreTemplateBuildFnTable, CoreTemplatePropertyKind,
     IntoTemplateProperty, TemplateBuildMethodFnMap, TemplateLanguage,
 };
-use crate::template_parser::{self, FunctionCallNode, TemplateParseResult};
+use crate::template_parser::{self, FunctionCallNode, TemplateParseError, TemplateParseResult};
 use crate::templater::{
-    self, IntoTemplate, PlainTextFormattedProperty, Template, TemplateFunction, TemplateProperty,
-    TemplatePropertyFn,
+    self, IntoTemplate, PlainTextFormattedProperty, Template, TemplateFormatter, TemplateProperty,
+    TemplatePropertyError, TemplatePropertyExt as _,
 };
-use crate::text_util;
+use crate::{revset_util, text_util};
 
 pub trait CommitTemplateLanguageExtension {
     fn build_fn_table<'repo>(&self) -> CommitTemplateBuildFnTable<'repo>;
+
+    fn build_cache_extensions(&self, extensions: &mut ExtensionsMap);
 }
 
 pub struct CommitTemplateLanguage<'repo> {
     repo: &'repo dyn Repo,
     workspace_id: WorkspaceId,
+    // RevsetParseContext doesn't borrow a repo, but we'll need 'repo lifetime
+    // anyway to capture it to evaluate dynamically-constructed user expression
+    // such as `revset("ancestors(" ++ commit_id ++ ")")`.
+    // TODO: Maybe refactor context structs? WorkspaceId is contained in
+    // RevsetParseContext for example.
+    revset_parse_context: RevsetParseContext<'repo>,
     id_prefix_context: &'repo IdPrefixContext,
     build_fn_table: CommitTemplateBuildFnTable<'repo>,
     keyword_cache: CommitKeywordCache,
+    cache_extensions: ExtensionsMap,
 }
 
 impl<'repo> CommitTemplateLanguage<'repo> {
@@ -58,32 +69,44 @@ impl<'repo> CommitTemplateLanguage<'repo> {
     pub fn new(
         repo: &'repo dyn Repo,
         workspace_id: &WorkspaceId,
+        revset_parse_context: RevsetParseContext<'repo>,
         id_prefix_context: &'repo IdPrefixContext,
         extension: Option<&dyn CommitTemplateLanguageExtension>,
     ) -> Self {
         let mut build_fn_table = CommitTemplateBuildFnTable::builtin();
+        let mut cache_extensions = ExtensionsMap::empty();
+
+        // TODO: Extension methods should be refactored to be plural, to support
+        // multiple extensions in a dynamic load environment
         if let Some(extension) = extension {
             build_fn_table.merge(extension.build_fn_table());
+            extension.build_cache_extensions(&mut cache_extensions);
         }
+
         CommitTemplateLanguage {
             repo,
             workspace_id: workspace_id.clone(),
+            revset_parse_context,
             id_prefix_context,
             build_fn_table,
             keyword_cache: CommitKeywordCache::default(),
+            cache_extensions,
         }
     }
 }
 
 impl<'repo> TemplateLanguage<'repo> for CommitTemplateLanguage<'repo> {
-    type Context = Commit;
     type Property = CommitTemplatePropertyKind<'repo>;
 
     template_builder::impl_core_wrap_property_fns!('repo, CommitTemplatePropertyKind::Core);
 
-    fn build_self(&self) -> Self::Property {
-        // Commit object is lightweight (a few Arc + CommitId)
-        self.wrap_commit(TemplatePropertyFn(|commit: &Commit| Ok(commit.clone())))
+    fn build_function(
+        &self,
+        build_ctx: &BuildContext<Self::Property>,
+        function: &FunctionCallNode,
+    ) -> TemplateParseResult<Self::Property> {
+        let table = &self.build_fn_table.core;
+        table.build_function(self, build_ctx, function)
     }
 
     fn build_method(
@@ -102,6 +125,14 @@ impl<'repo> TemplateLanguage<'repo> for CommitTemplateLanguage<'repo> {
                 let build = template_parser::lookup_method("Commit", table, function)?;
                 build(self, build_ctx, property, function)
             }
+            CommitTemplatePropertyKind::CommitOpt(property) => {
+                let table = &self.build_fn_table.commit_methods;
+                let build = template_parser::lookup_method("Commit", table, function)?;
+                let inner_property = property.and_then(|opt| {
+                    opt.ok_or_else(|| TemplatePropertyError("No commit available".into()))
+                });
+                build(self, build_ctx, Box::new(inner_property), function)
+            }
             CommitTemplatePropertyKind::CommitList(property) => {
                 // TODO: migrate to table?
                 template_builder::build_unformattable_list_method(
@@ -109,7 +140,7 @@ impl<'repo> TemplateLanguage<'repo> for CommitTemplateLanguage<'repo> {
                     build_ctx,
                     property,
                     function,
-                    |item| self.wrap_commit(item),
+                    Self::wrap_commit,
                 )
             }
             CommitTemplatePropertyKind::RefName(property) => {
@@ -117,6 +148,14 @@ impl<'repo> TemplateLanguage<'repo> for CommitTemplateLanguage<'repo> {
                 let build = template_parser::lookup_method("RefName", table, function)?;
                 build(self, build_ctx, property, function)
             }
+            CommitTemplatePropertyKind::RefNameOpt(property) => {
+                let table = &self.build_fn_table.ref_name_methods;
+                let build = template_parser::lookup_method("RefName", table, function)?;
+                let inner_property = property.and_then(|opt| {
+                    opt.ok_or_else(|| TemplatePropertyError("No RefName available".into()))
+                });
+                build(self, build_ctx, Box::new(inner_property), function)
+            }
             CommitTemplatePropertyKind::RefNameList(property) => {
                 // TODO: migrate to table?
                 template_builder::build_formattable_list_method(
@@ -124,7 +163,7 @@ impl<'repo> TemplateLanguage<'repo> for CommitTemplateLanguage<'repo> {
                     build_ctx,
                     property,
                     function,
-                    |item| self.wrap_ref_name(item),
+                    Self::wrap_ref_name,
                 )
             }
             CommitTemplatePropertyKind::CommitOrChangeId(property) => {
@@ -144,7 +183,7 @@ impl<'repo> TemplateLanguage<'repo> for CommitTemplateLanguage<'repo> {
 // If we need to add multiple languages that support Commit types, this can be
 // turned into a trait which extends TemplateLanguage.
 impl<'repo> CommitTemplateLanguage<'repo> {
-    pub fn repo(&self) -> &dyn Repo {
+    pub fn repo(&self) -> &'repo dyn Repo {
         self.repo
     }
 
@@ -156,90 +195,102 @@ impl<'repo> CommitTemplateLanguage<'repo> {
         &self.keyword_cache
     }
 
+    pub fn cache_extension<T: Any>(&self) -> Option<&T> {
+        self.cache_extensions.get::<T>()
+    }
+
     pub fn wrap_commit(
-        &self,
-        property: impl TemplateProperty<Commit, Output = Commit> + 'repo,
+        property: impl TemplateProperty<Output = Commit> + 'repo,
     ) -> CommitTemplatePropertyKind<'repo> {
         CommitTemplatePropertyKind::Commit(Box::new(property))
     }
 
+    pub fn wrap_commit_opt(
+        property: impl TemplateProperty<Output = Option<Commit>> + 'repo,
+    ) -> CommitTemplatePropertyKind<'repo> {
+        CommitTemplatePropertyKind::CommitOpt(Box::new(property))
+    }
+
     pub fn wrap_commit_list(
-        &self,
-        property: impl TemplateProperty<Commit, Output = Vec<Commit>> + 'repo,
+        property: impl TemplateProperty<Output = Vec<Commit>> + 'repo,
     ) -> CommitTemplatePropertyKind<'repo> {
         CommitTemplatePropertyKind::CommitList(Box::new(property))
     }
 
     pub fn wrap_ref_name(
-        &self,
-        property: impl TemplateProperty<Commit, Output = RefName> + 'repo,
+        property: impl TemplateProperty<Output = RefName> + 'repo,
     ) -> CommitTemplatePropertyKind<'repo> {
         CommitTemplatePropertyKind::RefName(Box::new(property))
     }
 
+    pub fn wrap_ref_name_opt(
+        property: impl TemplateProperty<Output = Option<RefName>> + 'repo,
+    ) -> CommitTemplatePropertyKind<'repo> {
+        CommitTemplatePropertyKind::RefNameOpt(Box::new(property))
+    }
+
     pub fn wrap_ref_name_list(
-        &self,
-        property: impl TemplateProperty<Commit, Output = Vec<RefName>> + 'repo,
+        property: impl TemplateProperty<Output = Vec<RefName>> + 'repo,
     ) -> CommitTemplatePropertyKind<'repo> {
         CommitTemplatePropertyKind::RefNameList(Box::new(property))
     }
 
     pub fn wrap_commit_or_change_id(
-        &self,
-        property: impl TemplateProperty<Commit, Output = CommitOrChangeId> + 'repo,
+        property: impl TemplateProperty<Output = CommitOrChangeId> + 'repo,
     ) -> CommitTemplatePropertyKind<'repo> {
         CommitTemplatePropertyKind::CommitOrChangeId(Box::new(property))
     }
 
     pub fn wrap_shortest_id_prefix(
-        &self,
-        property: impl TemplateProperty<Commit, Output = ShortestIdPrefix> + 'repo,
+        property: impl TemplateProperty<Output = ShortestIdPrefix> + 'repo,
     ) -> CommitTemplatePropertyKind<'repo> {
         CommitTemplatePropertyKind::ShortestIdPrefix(Box::new(property))
     }
 }
 
 pub enum CommitTemplatePropertyKind<'repo> {
-    Core(CoreTemplatePropertyKind<'repo, Commit>),
-    Commit(Box<dyn TemplateProperty<Commit, Output = Commit> + 'repo>),
-    CommitList(Box<dyn TemplateProperty<Commit, Output = Vec<Commit>> + 'repo>),
-    RefName(Box<dyn TemplateProperty<Commit, Output = RefName> + 'repo>),
-    RefNameList(Box<dyn TemplateProperty<Commit, Output = Vec<RefName>> + 'repo>),
-    CommitOrChangeId(Box<dyn TemplateProperty<Commit, Output = CommitOrChangeId> + 'repo>),
-    ShortestIdPrefix(Box<dyn TemplateProperty<Commit, Output = ShortestIdPrefix> + 'repo>),
+    Core(CoreTemplatePropertyKind<'repo>),
+    Commit(Box<dyn TemplateProperty<Output = Commit> + 'repo>),
+    CommitOpt(Box<dyn TemplateProperty<Output = Option<Commit>> + 'repo>),
+    CommitList(Box<dyn TemplateProperty<Output = Vec<Commit>> + 'repo>),
+    RefName(Box<dyn TemplateProperty<Output = RefName> + 'repo>),
+    RefNameOpt(Box<dyn TemplateProperty<Output = Option<RefName>> + 'repo>),
+    RefNameList(Box<dyn TemplateProperty<Output = Vec<RefName>> + 'repo>),
+    CommitOrChangeId(Box<dyn TemplateProperty<Output = CommitOrChangeId> + 'repo>),
+    ShortestIdPrefix(Box<dyn TemplateProperty<Output = ShortestIdPrefix> + 'repo>),
 }
 
-impl<'repo> IntoTemplateProperty<'repo, Commit> for CommitTemplatePropertyKind<'repo> {
-    fn try_into_boolean(self) -> Option<Box<dyn TemplateProperty<Commit, Output = bool> + 'repo>> {
+impl<'repo> IntoTemplateProperty<'repo> for CommitTemplatePropertyKind<'repo> {
+    fn try_into_boolean(self) -> Option<Box<dyn TemplateProperty<Output = bool> + 'repo>> {
         match self {
             CommitTemplatePropertyKind::Core(property) => property.try_into_boolean(),
             CommitTemplatePropertyKind::Commit(_) => None,
+            CommitTemplatePropertyKind::CommitOpt(property) => {
+                Some(Box::new(property.map(|opt| opt.is_some())))
+            }
             CommitTemplatePropertyKind::CommitList(property) => {
-                Some(Box::new(TemplateFunction::new(property, |l| {
-                    Ok(!l.is_empty())
-                })))
+                Some(Box::new(property.map(|l| !l.is_empty())))
             }
             CommitTemplatePropertyKind::RefName(_) => None,
+            CommitTemplatePropertyKind::RefNameOpt(property) => {
+                Some(Box::new(property.map(|opt| opt.is_some())))
+            }
             CommitTemplatePropertyKind::RefNameList(property) => {
-                Some(Box::new(TemplateFunction::new(property, |l| {
-                    Ok(!l.is_empty())
-                })))
+                Some(Box::new(property.map(|l| !l.is_empty())))
             }
             CommitTemplatePropertyKind::CommitOrChangeId(_) => None,
             CommitTemplatePropertyKind::ShortestIdPrefix(_) => None,
         }
     }
 
-    fn try_into_integer(self) -> Option<Box<dyn TemplateProperty<Commit, Output = i64> + 'repo>> {
+    fn try_into_integer(self) -> Option<Box<dyn TemplateProperty<Output = i64> + 'repo>> {
         match self {
             CommitTemplatePropertyKind::Core(property) => property.try_into_integer(),
             _ => None,
         }
     }
 
-    fn try_into_plain_text(
-        self,
-    ) -> Option<Box<dyn TemplateProperty<Commit, Output = String> + 'repo>> {
+    fn try_into_plain_text(self) -> Option<Box<dyn TemplateProperty<Output = String> + 'repo>> {
         match self {
             CommitTemplatePropertyKind::Core(property) => property.try_into_plain_text(),
             _ => {
@@ -249,12 +300,14 @@ impl<'repo> IntoTemplateProperty<'repo, Commit> for CommitTemplatePropertyKind<'
         }
     }
 
-    fn try_into_template(self) -> Option<Box<dyn Template<Commit> + 'repo>> {
+    fn try_into_template(self) -> Option<Box<dyn Template + 'repo>> {
         match self {
             CommitTemplatePropertyKind::Core(property) => property.try_into_template(),
             CommitTemplatePropertyKind::Commit(_) => None,
+            CommitTemplatePropertyKind::CommitOpt(_) => None,
             CommitTemplatePropertyKind::CommitList(_) => None,
             CommitTemplatePropertyKind::RefName(property) => Some(property.into_template()),
+            CommitTemplatePropertyKind::RefNameOpt(property) => Some(property.into_template()),
             CommitTemplatePropertyKind::RefNameList(property) => Some(property.into_template()),
             CommitTemplatePropertyKind::CommitOrChangeId(property) => {
                 Some(property.into_template())
@@ -350,60 +403,59 @@ impl CommitKeywordCache {
 }
 
 fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Commit> {
+    type L<'repo> = CommitTemplateLanguage<'repo>;
     // Not using maplit::hashmap!{} or custom declarative macro here because
     // code completion inside macro is quite restricted.
     let mut map = CommitTemplateBuildMethodFnMap::<Commit>::new();
     map.insert(
         "description",
-        |language, _build_ctx, self_property, function| {
+        |_language, _build_ctx, self_property, function| {
             template_parser::expect_no_arguments(function)?;
-            let out_property = TemplateFunction::new(self_property, |commit| {
-                Ok(text_util::complete_newline(commit.description()))
-            });
-            Ok(language.wrap_string(out_property))
+            let out_property =
+                self_property.map(|commit| text_util::complete_newline(commit.description()));
+            Ok(L::wrap_string(out_property))
         },
     );
     map.insert(
         "change_id",
-        |language, _build_ctx, self_property, function| {
+        |_language, _build_ctx, self_property, function| {
             template_parser::expect_no_arguments(function)?;
-            let out_property = TemplateFunction::new(self_property, |commit| {
-                Ok(CommitOrChangeId::Change(commit.change_id().to_owned()))
-            });
-            Ok(language.wrap_commit_or_change_id(out_property))
+            let out_property =
+                self_property.map(|commit| CommitOrChangeId::Change(commit.change_id().to_owned()));
+            Ok(L::wrap_commit_or_change_id(out_property))
         },
     );
     map.insert(
         "commit_id",
-        |language, _build_ctx, self_property, function| {
+        |_language, _build_ctx, self_property, function| {
             template_parser::expect_no_arguments(function)?;
-            let out_property = TemplateFunction::new(self_property, |commit| {
-                Ok(CommitOrChangeId::Commit(commit.id().to_owned()))
-            });
-            Ok(language.wrap_commit_or_change_id(out_property))
+            let out_property =
+                self_property.map(|commit| CommitOrChangeId::Commit(commit.id().to_owned()));
+            Ok(L::wrap_commit_or_change_id(out_property))
         },
     );
     map.insert(
         "parents",
-        |language, _build_ctx, self_property, function| {
+        |_language, _build_ctx, self_property, function| {
             template_parser::expect_no_arguments(function)?;
-            let out_property = TemplateFunction::new(self_property, |commit| Ok(commit.parents()));
-            Ok(language.wrap_commit_list(out_property))
+            let out_property = self_property.map(|commit| commit.parents());
+            Ok(L::wrap_commit_list(out_property))
+        },
+    );
+    map.insert(
+        "author",
+        |_language, _build_ctx, self_property, function| {
+            template_parser::expect_no_arguments(function)?;
+            let out_property = self_property.map(|commit| commit.author().clone());
+            Ok(L::wrap_signature(out_property))
         },
     );
-    map.insert("author", |language, _build_ctx, self_property, function| {
-        template_parser::expect_no_arguments(function)?;
-        let out_property =
-            TemplateFunction::new(self_property, |commit| Ok(commit.author().clone()));
-        Ok(language.wrap_signature(out_property))
-    });
     map.insert(
         "committer",
-        |language, _build_ctx, self_property, function| {
+        |_language, _build_ctx, self_property, function| {
             template_parser::expect_no_arguments(function)?;
-            let out_property =
-                TemplateFunction::new(self_property, |commit| Ok(commit.committer().clone()));
-            Ok(language.wrap_signature(out_property))
+            let out_property = self_property.map(|commit| commit.committer().clone());
+            Ok(L::wrap_signature(out_property))
         },
     );
     map.insert(
@@ -411,10 +463,8 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
         |language, _build_ctx, self_property, function| {
             template_parser::expect_no_arguments(function)?;
             let repo = language.repo;
-            let out_property = TemplateFunction::new(self_property, |commit| {
-                Ok(extract_working_copies(repo, &commit))
-            });
-            Ok(language.wrap_string(out_property))
+            let out_property = self_property.map(|commit| extract_working_copies(repo, &commit));
+            Ok(L::wrap_string(out_property))
         },
     );
     map.insert(
@@ -423,10 +473,10 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
             template_parser::expect_no_arguments(function)?;
             let repo = language.repo;
             let workspace_id = language.workspace_id.clone();
-            let out_property = TemplateFunction::new(self_property, move |commit| {
-                Ok(Some(commit.id()) == repo.view().get_wc_commit_id(&workspace_id))
+            let out_property = self_property.map(move |commit| {
+                Some(commit.id()) == repo.view().get_wc_commit_id(&workspace_id)
             });
-            Ok(language.wrap_boolean(out_property))
+            Ok(L::wrap_boolean(out_property))
         },
     );
     map.insert(
@@ -434,15 +484,15 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
         |language, _build_ctx, self_property, function| {
             template_parser::expect_no_arguments(function)?;
             let index = language.keyword_cache.branches_index(language.repo).clone();
-            let out_property = TemplateFunction::new(self_property, move |commit| {
-                Ok(index
+            let out_property = self_property.map(move |commit| {
+                index
                     .get(commit.id())
                     .iter()
                     .filter(|ref_name| ref_name.is_local() || !ref_name.synced)
                     .cloned()
-                    .collect())
+                    .collect()
             });
-            Ok(language.wrap_ref_name_list(out_property))
+            Ok(L::wrap_ref_name_list(out_property))
         },
     );
     map.insert(
@@ -450,15 +500,15 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
         |language, _build_ctx, self_property, function| {
             template_parser::expect_no_arguments(function)?;
             let index = language.keyword_cache.branches_index(language.repo).clone();
-            let out_property = TemplateFunction::new(self_property, move |commit| {
-                Ok(index
+            let out_property = self_property.map(move |commit| {
+                index
                     .get(commit.id())
                     .iter()
                     .filter(|ref_name| ref_name.is_local())
                     .cloned()
-                    .collect())
+                    .collect()
             });
-            Ok(language.wrap_ref_name_list(out_property))
+            Ok(L::wrap_ref_name_list(out_property))
         },
     );
     map.insert(
@@ -466,34 +516,30 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
         |language, _build_ctx, self_property, function| {
             template_parser::expect_no_arguments(function)?;
             let index = language.keyword_cache.branches_index(language.repo).clone();
-            let out_property = TemplateFunction::new(self_property, move |commit| {
-                Ok(index
+            let out_property = self_property.map(move |commit| {
+                index
                     .get(commit.id())
                     .iter()
                     .filter(|ref_name| ref_name.is_remote())
                     .cloned()
-                    .collect())
+                    .collect()
             });
-            Ok(language.wrap_ref_name_list(out_property))
+            Ok(L::wrap_ref_name_list(out_property))
         },
     );
     map.insert("tags", |language, _build_ctx, self_property, function| {
         template_parser::expect_no_arguments(function)?;
         let index = language.keyword_cache.tags_index(language.repo).clone();
-        let out_property = TemplateFunction::new(self_property, move |commit| {
-            Ok(index.get(commit.id()).to_vec())
-        });
-        Ok(language.wrap_ref_name_list(out_property))
+        let out_property = self_property.map(move |commit| index.get(commit.id()).to_vec());
+        Ok(L::wrap_ref_name_list(out_property))
     });
     map.insert(
         "git_refs",
         |language, _build_ctx, self_property, function| {
             template_parser::expect_no_arguments(function)?;
             let index = language.keyword_cache.git_refs_index(language.repo).clone();
-            let out_property = TemplateFunction::new(self_property, move |commit| {
-                Ok(index.get(commit.id()).to_vec())
-            });
-            Ok(language.wrap_ref_name_list(out_property))
+            let out_property = self_property.map(move |commit| index.get(commit.id()).to_vec());
+            Ok(L::wrap_ref_name_list(out_property))
         },
     );
     map.insert(
@@ -501,9 +547,8 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
         |language, _build_ctx, self_property, function| {
             template_parser::expect_no_arguments(function)?;
             let repo = language.repo;
-            let out_property =
-                TemplateFunction::new(self_property, |commit| Ok(extract_git_head(repo, &commit)));
-            Ok(language.wrap_ref_name_list(out_property))
+            let out_property = self_property.map(|commit| extract_git_head(repo, &commit));
+            Ok(L::wrap_ref_name_opt(out_property))
         },
     );
     map.insert(
@@ -511,51 +556,58 @@ fn builtin_commit_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, Comm
         |language, _build_ctx, self_property, function| {
             template_parser::expect_no_arguments(function)?;
             let repo = language.repo;
-            let out_property = TemplateFunction::new(self_property, |commit| {
+            let out_property = self_property.map(|commit| {
                 // The given commit could be hidden in e.g. obslog.
                 let maybe_entries = repo.resolve_change_id(commit.change_id());
-                Ok(maybe_entries.map_or(0, |entries| entries.len()) > 1)
+                maybe_entries.map_or(0, |entries| entries.len()) > 1
             });
-            Ok(language.wrap_boolean(out_property))
+            Ok(L::wrap_boolean(out_property))
         },
     );
     map.insert("hidden", |language, _build_ctx, self_property, function| {
         template_parser::expect_no_arguments(function)?;
         let repo = language.repo;
-        let out_property = TemplateFunction::new(self_property, |commit| {
+        let out_property = self_property.map(|commit| {
             let maybe_entries = repo.resolve_change_id(commit.change_id());
-            Ok(maybe_entries.map_or(true, |entries| !entries.contains(commit.id())))
+            maybe_entries.map_or(true, |entries| !entries.contains(commit.id()))
         });
-        Ok(language.wrap_boolean(out_property))
+        Ok(L::wrap_boolean(out_property))
     });
     map.insert(
-        "conflict",
+        "immutable",
         |language, _build_ctx, self_property, function| {
             template_parser::expect_no_arguments(function)?;
-            let out_property =
-                TemplateFunction::new(self_property, |commit| Ok(commit.has_conflict()?));
-            Ok(language.wrap_boolean(out_property))
+            let revset = evaluate_immutable_revset(language, function.name_span)?;
+            let is_immutable = revset.containing_fn();
+            let out_property = self_property.map(move |commit| is_immutable(commit.id()));
+            Ok(L::wrap_boolean(out_property))
+        },
+    );
+    map.insert(
+        "conflict",
+        |_language, _build_ctx, self_property, function| {
+            template_parser::expect_no_arguments(function)?;
+            let out_property = self_property.and_then(|commit| Ok(commit.has_conflict()?));
+            Ok(L::wrap_boolean(out_property))
         },
     );
     map.insert("empty", |language, _build_ctx, self_property, function| {
         template_parser::expect_no_arguments(function)?;
         let repo = language.repo;
-        let out_property = TemplateFunction::new(self_property, |commit| {
+        let out_property = self_property.and_then(|commit| {
             if let [parent] = &commit.parents()[..] {
                 return Ok(parent.tree_id() == commit.tree_id());
             }
             let parent_tree = rewrite::merge_commit_trees(repo, &commit.parents())?;
             Ok(*commit.tree_id() == parent_tree.id())
         });
-        Ok(language.wrap_boolean(out_property))
+        Ok(L::wrap_boolean(out_property))
     });
     map.insert("root", |language, _build_ctx, self_property, function| {
         template_parser::expect_no_arguments(function)?;
         let repo = language.repo;
-        let out_property = TemplateFunction::new(self_property, |commit| {
-            Ok(commit.id() == repo.store().root_commit_id())
-        });
-        Ok(language.wrap_boolean(out_property))
+        let out_property = self_property.map(|commit| commit.id() == repo.store().root_commit_id());
+        Ok(L::wrap_boolean(out_property))
     });
     map
 }
@@ -575,6 +627,25 @@ fn extract_working_copies(repo: &dyn Repo, commit: &Commit) -> String {
     names.join(" ")
 }
 
+fn evaluate_immutable_revset<'repo>(
+    language: &CommitTemplateLanguage<'repo>,
+    span: pest::Span<'_>,
+) -> Result<Box<dyn Revset + 'repo>, TemplateParseError> {
+    let repo = language.repo;
+    // Alternatively, a negated (i.e. visible mutable) set could be computed.
+    // It's usually smaller than the immutable set. The revset engine can also
+    // optimize "::<recent_heads>" query to use bitset-based implementation.
+    let expression = revset_util::parse_immutable_expression(&language.revset_parse_context)
+        .map_err(|err| {
+            TemplateParseError::expression("Failed to parse revset", span).with_source(err)
+        })?;
+    let symbol_resolver = revset_util::default_symbol_resolver(repo, language.id_prefix_context);
+    let revset = revset_util::evaluate(repo, &symbol_resolver, expression).map_err(|err| {
+        TemplateParseError::expression("Failed to evaluate revset", span).with_source(err)
+    })?;
+    Ok(revset)
+}
+
 /// Branch or tag name with metadata.
 #[derive(Clone, Debug, Eq, PartialEq)]
 pub struct RefName {
@@ -599,8 +670,8 @@ impl RefName {
     }
 }
 
-impl Template<()> for RefName {
-    fn format(&self, _: &(), formatter: &mut dyn Formatter) -> io::Result<()> {
+impl Template for RefName {
+    fn format(&self, formatter: &mut TemplateFormatter) -> io::Result<()> {
         write!(formatter.labeled("name"), "{}", self.name)?;
         if let Some(remote) = &self.remote {
             write!(formatter, "@")?;
@@ -617,28 +688,30 @@ impl Template<()> for RefName {
     }
 }
 
-impl Template<()> for Vec<RefName> {
-    fn format(&self, _: &(), formatter: &mut dyn Formatter) -> io::Result<()> {
-        templater::format_joined(&(), formatter, self, " ")
+impl Template for Vec<RefName> {
+    fn format(&self, formatter: &mut TemplateFormatter) -> io::Result<()> {
+        templater::format_joined(formatter, self, " ")
     }
 }
 
 fn builtin_ref_name_methods<'repo>() -> CommitTemplateBuildMethodFnMap<'repo, RefName> {
+    type L<'repo> = CommitTemplateLanguage<'repo>;
     // Not using maplit::hashmap!{} or custom declarative macro here because
     // code completion inside macro is quite restricted.
     let mut map = CommitTemplateBuildMethodFnMap::<RefName>::new();
-    map.insert("name", |language, _build_ctx, self_property, function| {
+    map.insert("name", |_language, _build_ctx, self_property, function| {
         template_parser::expect_no_arguments(function)?;
-        let out_property = TemplateFunction::new(self_property, |ref_name| Ok(ref_name.name));
-        Ok(language.wrap_string(out_property))
-    });
-    map.insert("remote", |language, _build_ctx, self_property, function| {
-        template_parser::expect_no_arguments(function)?;
-        let out_property = TemplateFunction::new(self_property, |ref_name| {
-            Ok(ref_name.remote.unwrap_or_default())
-        });
-        Ok(language.wrap_string(out_property))
+        let out_property = self_property.map(|ref_name| ref_name.name);
+        Ok(L::wrap_string(out_property))
     });
+    map.insert(
+        "remote",
+        |_language, _build_ctx, self_property, function| {
+            template_parser::expect_no_arguments(function)?;
+            let out_property = self_property.map(|ref_name| ref_name.remote.unwrap_or_default());
+            Ok(L::wrap_string(out_property))
+        },
+    );
     map
 }
 
@@ -710,20 +783,16 @@ fn build_ref_names_index<'a>(
     index
 }
 
-// TODO: maybe add option or nullable type?
-fn extract_git_head(repo: &dyn Repo, commit: &Commit) -> Vec<RefName> {
+fn extract_git_head(repo: &dyn Repo, commit: &Commit) -> Option<RefName> {
     let target = repo.view().git_head();
-    if target.added_ids().contains(commit.id()) {
-        let ref_name = RefName {
+    target.added_ids().contains(commit.id()).then(|| {
+        RefName {
             name: "HEAD".to_owned(),
             remote: Some(git::REMOTE_NAME_FOR_LOCAL_GIT_REPO.to_owned()),
             conflict: target.has_conflict(),
             synced: false, // has no local counterpart
-        };
-        vec![ref_name]
-    } else {
-        vec![]
-    }
+        }
+    })
 }
 
 #[derive(Clone, Debug, Eq, PartialEq)]
@@ -769,14 +838,15 @@ impl CommitOrChangeId {
     }
 }
 
-impl Template<()> for CommitOrChangeId {
-    fn format(&self, _: &(), formatter: &mut dyn Formatter) -> io::Result<()> {
-        formatter.write_str(&self.hex())
+impl Template for CommitOrChangeId {
+    fn format(&self, formatter: &mut TemplateFormatter) -> io::Result<()> {
+        write!(formatter, "{}", self.hex())
     }
 }
 
 fn builtin_commit_or_change_id_methods<'repo>(
 ) -> CommitTemplateBuildMethodFnMap<'repo, CommitOrChangeId> {
+    type L<'repo> = CommitTemplateLanguage<'repo>;
     // Not using maplit::hashmap!{} or custom declarative macro here because
     // code completion inside macro is quite restricted.
     let mut map = CommitTemplateBuildMethodFnMap::<CommitOrChangeId>::new();
@@ -785,10 +855,9 @@ fn builtin_commit_or_change_id_methods<'repo>(
         let len_property = len_node
             .map(|node| template_builder::expect_usize_expression(language, build_ctx, node))
             .transpose()?;
-        let out_property = TemplateFunction::new((self_property, len_property), |(id, len)| {
-            Ok(id.short(len.unwrap_or(12)))
-        });
-        Ok(language.wrap_string(out_property))
+        let out_property =
+            (self_property, len_property).map(|(id, len)| id.short(len.unwrap_or(12)));
+        Ok(L::wrap_string(out_property))
     });
     map.insert(
         "shortest",
@@ -798,10 +867,9 @@ fn builtin_commit_or_change_id_methods<'repo>(
             let len_property = len_node
                 .map(|node| template_builder::expect_usize_expression(language, build_ctx, node))
                 .transpose()?;
-            let out_property = TemplateFunction::new((self_property, len_property), |(id, len)| {
-                Ok(id.shortest(language.repo, id_prefix_context, len.unwrap_or(0)))
-            });
-            Ok(language.wrap_shortest_id_prefix(out_property))
+            let out_property = (self_property, len_property)
+                .map(|(id, len)| id.shortest(language.repo, id_prefix_context, len.unwrap_or(0)));
+            Ok(L::wrap_shortest_id_prefix(out_property))
         },
     );
     map
@@ -812,10 +880,11 @@ pub struct ShortestIdPrefix {
     pub rest: String,
 }
 
-impl Template<()> for ShortestIdPrefix {
-    fn format(&self, _: &(), formatter: &mut dyn Formatter) -> io::Result<()> {
-        formatter.with_label("prefix", |fmt| fmt.write_str(&self.prefix))?;
-        formatter.with_label("rest", |fmt| fmt.write_str(&self.rest))
+impl Template for ShortestIdPrefix {
+    fn format(&self, formatter: &mut TemplateFormatter) -> io::Result<()> {
+        write!(formatter.labeled("prefix"), "{}", self.prefix)?;
+        write!(formatter.labeled("rest"), "{}", self.rest)?;
+        Ok(())
     }
 }
 
@@ -836,28 +905,32 @@ impl ShortestIdPrefix {
 
 fn builtin_shortest_id_prefix_methods<'repo>(
 ) -> CommitTemplateBuildMethodFnMap<'repo, ShortestIdPrefix> {
+    type L<'repo> = CommitTemplateLanguage<'repo>;
     // Not using maplit::hashmap!{} or custom declarative macro here because
     // code completion inside macro is quite restricted.
     let mut map = CommitTemplateBuildMethodFnMap::<ShortestIdPrefix>::new();
-    map.insert("prefix", |language, _build_ctx, self_property, function| {
+    map.insert(
+        "prefix",
+        |_language, _build_ctx, self_property, function| {
+            template_parser::expect_no_arguments(function)?;
+            let out_property = self_property.map(|id| id.prefix);
+            Ok(L::wrap_string(out_property))
+        },
+    );
+    map.insert("rest", |_language, _build_ctx, self_property, function| {
         template_parser::expect_no_arguments(function)?;
-        let out_property = TemplateFunction::new(self_property, |id| Ok(id.prefix));
-        Ok(language.wrap_string(out_property))
+        let out_property = self_property.map(|id| id.rest);
+        Ok(L::wrap_string(out_property))
     });
-    map.insert("rest", |language, _build_ctx, self_property, function| {
+    map.insert("upper", |_language, _build_ctx, self_property, function| {
         template_parser::expect_no_arguments(function)?;
-        let out_property = TemplateFunction::new(self_property, |id| Ok(id.rest));
-        Ok(language.wrap_string(out_property))
+        let out_property = self_property.map(|id| id.to_upper());
+        Ok(L::wrap_shortest_id_prefix(out_property))
     });
-    map.insert("upper", |language, _build_ctx, self_property, function| {
+    map.insert("lower", |_language, _build_ctx, self_property, function| {
         template_parser::expect_no_arguments(function)?;
-        let out_property = TemplateFunction::new(self_property, |id| Ok(id.to_upper()));
-        Ok(language.wrap_shortest_id_prefix(out_property))
-    });
-    map.insert("lower", |language, _build_ctx, self_property, function| {
-        template_parser::expect_no_arguments(function)?;
-        let out_property = TemplateFunction::new(self_property, |id| Ok(id.to_lower()));
-        Ok(language.wrap_shortest_id_prefix(out_property))
+        let out_property = self_property.map(|id| id.to_lower());
+        Ok(L::wrap_shortest_id_prefix(out_property))
     });
     map
 }
