@@ -13,8 +13,7 @@
 // limitations under the License.
 
 use std::collections::HashMap;
-use std::num::ParseIntError;
-use std::{error, fmt, iter, mem};
+use std::{error, fmt, mem};
 
 use itertools::Itertools as _;
 use once_cell::sync::Lazy;
@@ -64,19 +63,18 @@ impl Rule {
 
 pub type TemplateParseResult<T> = Result<T, TemplateParseError>;
 
-#[derive(Clone, Debug)]
+#[derive(Debug, Error)]
+#[error("{pest_error}")]
 pub struct TemplateParseError {
     kind: TemplateParseErrorKind,
     pest_error: Box<pest::error::Error<Rule>>,
-    origin: Option<Box<TemplateParseError>>,
+    source: Option<Box<dyn error::Error + Send + Sync>>,
 }
 
 #[derive(Clone, Debug, Eq, Error, PartialEq)]
 pub enum TemplateParseErrorKind {
     #[error("Syntax error")]
     SyntaxError,
-    #[error("Invalid integer literal")]
-    ParseIntError(#[source] ParseIntError),
     #[error(r#"Keyword "{name}" doesn't exist"#)]
     NoSuchKeyword {
         name: String,
@@ -98,7 +96,7 @@ pub enum TemplateParseErrorKind {
     #[error("Redefinition of function parameter")]
     RedefinedFunctionParameter,
     #[error("{0}")]
-    UnexpectedExpression(String),
+    Expression(String),
     #[error(r#"Alias "{0}" cannot be expanded"#)]
     BadAliasExpansion(String),
     #[error(r#"Alias "{0}" expanded recursively"#)]
@@ -107,7 +105,7 @@ pub enum TemplateParseErrorKind {
 
 impl TemplateParseError {
     pub fn with_span(kind: TemplateParseErrorKind, span: pest::Span<'_>) -> Self {
-        let message = iter::successors(Some(&kind as &dyn error::Error), |e| e.source()).join(": ");
+        let message = kind.to_string();
         let pest_error = Box::new(pest::error::Error::new_from_span(
             pest::error::ErrorVariant::CustomError { message },
             span,
@@ -115,36 +113,13 @@ impl TemplateParseError {
         TemplateParseError {
             kind,
             pest_error,
-            origin: None,
+            source: None,
         }
     }
 
-    pub fn with_span_and_origin(
-        kind: TemplateParseErrorKind,
-        span: pest::Span<'_>,
-        origin: Self,
-    ) -> Self {
-        let message = iter::successors(Some(&kind as &dyn error::Error), |e| e.source()).join(": ");
-        let pest_error = Box::new(pest::error::Error::new_from_span(
-            pest::error::ErrorVariant::CustomError { message },
-            span,
-        ));
-        TemplateParseError {
-            kind,
-            pest_error,
-            origin: Some(Box::new(origin)),
-        }
-    }
-
-    // TODO: migrate callers to something like lookup_method()
-    pub(crate) fn no_such_function(function: &FunctionCallNode) -> Self {
-        TemplateParseError::with_span(
-            TemplateParseErrorKind::NoSuchFunction {
-                name: function.name.to_owned(),
-                candidates: vec![],
-            },
-            function.name_span,
-        )
+    pub fn with_source(mut self, source: impl Into<Box<dyn error::Error + Send + Sync>>) -> Self {
+        self.source = Some(source.into());
+        self
     }
 
     // TODO: migrate all callers to table-based lookup_method()
@@ -174,22 +149,20 @@ impl TemplateParseError {
 
     pub fn expected_type(type_name: &str, span: pest::Span<'_>) -> Self {
         let message = format!(r#"Expected expression of type "{type_name}""#);
-        TemplateParseError::unexpected_expression(message, span)
+        TemplateParseError::expression(message, span)
     }
 
-    pub fn unexpected_expression(message: impl Into<String>, span: pest::Span<'_>) -> Self {
-        TemplateParseError::with_span(
-            TemplateParseErrorKind::UnexpectedExpression(message.into()),
-            span,
-        )
+    /// Some other expression error.
+    pub fn expression(message: impl Into<String>, span: pest::Span<'_>) -> Self {
+        TemplateParseError::with_span(TemplateParseErrorKind::Expression(message.into()), span)
     }
 
     pub fn within_alias_expansion(self, id: TemplateAliasId<'_>, span: pest::Span<'_>) -> Self {
-        TemplateParseError::with_span_and_origin(
+        TemplateParseError::with_span(
             TemplateParseErrorKind::BadAliasExpansion(id.to_string()),
             span,
-            self,
         )
+        .with_source(self)
     }
 
     /// If this is a `NoSuchKeyword` error, expands the candidates list with the
@@ -236,7 +209,7 @@ impl TemplateParseError {
 
     /// Original parsing error which typically occurred in an alias expression.
     pub fn origin(&self) -> Option<&Self> {
-        self.origin.as_deref()
+        self.source.as_ref().and_then(|e| e.downcast_ref())
     }
 }
 
@@ -245,27 +218,7 @@ impl From<pest::error::Error<Rule>> for TemplateParseError {
         TemplateParseError {
             kind: TemplateParseErrorKind::SyntaxError,
             pest_error: Box::new(rename_rules_in_pest_error(err)),
-            origin: None,
-        }
-    }
-}
-
-impl fmt::Display for TemplateParseError {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        self.pest_error.fmt(f)
-    }
-}
-
-impl error::Error for TemplateParseError {
-    fn source(&self) -> Option<&(dyn error::Error + 'static)> {
-        if let Some(e) = self.origin() {
-            return Some(e as &dyn error::Error);
-        }
-        match &self.kind {
-            // SyntaxError is a wrapper for pest::error::Error.
-            TemplateParseErrorKind::SyntaxError => Some(&self.pest_error as &dyn error::Error),
-            // Otherwise the kind represents this error.
-            e => e.source(),
+            source: None,
         }
     }
 }
@@ -358,10 +311,7 @@ fn parse_identifier_name(pair: Pair<Rule>) -> TemplateParseResult<&str> {
     if let ExpressionKind::Identifier(name) = parse_identifier_or_literal(pair) {
         Ok(name)
     } else {
-        Err(TemplateParseError::unexpected_expression(
-            "Expected identifier",
-            span,
-        ))
+        Err(TemplateParseError::expression("Expected identifier", span))
     }
 }
 
@@ -453,7 +403,7 @@ fn parse_term_node(pair: Pair<Rule>) -> TemplateParseResult<ExpressionNode> {
         }
         Rule::integer_literal => {
             let value = expr.as_str().parse().map_err(|err| {
-                TemplateParseError::with_span(TemplateParseErrorKind::ParseIntError(err), span)
+                TemplateParseError::expression("Invalid integer literal", span).with_source(err)
             })?;
             ExpressionNode::new(ExpressionKind::Integer(value), span)
         }
@@ -882,7 +832,7 @@ pub fn expect_string_literal_with<'a, 'i, T>(
         | ExpressionKind::Concat(_)
         | ExpressionKind::FunctionCall(_)
         | ExpressionKind::MethodCall(_)
-        | ExpressionKind::Lambda(_) => Err(TemplateParseError::unexpected_expression(
+        | ExpressionKind::Lambda(_) => Err(TemplateParseError::expression(
             "Expected string literal",
             node.span,
         )),
@@ -906,7 +856,7 @@ pub fn expect_lambda_with<'a, 'i, T>(
         | ExpressionKind::Binary(..)
         | ExpressionKind::Concat(_)
         | ExpressionKind::FunctionCall(_)
-        | ExpressionKind::MethodCall(_) => Err(TemplateParseError::unexpected_expression(
+        | ExpressionKind::MethodCall(_) => Err(TemplateParseError::expression(
             "Expected lambda expression",
             node.span,
         )),
@@ -916,6 +866,25 @@ pub fn expect_lambda_with<'a, 'i, T>(
     }
 }
 
+/// Looks up `table` by the given function name.
+pub fn lookup_function<'a, V>(
+    table: &'a HashMap<&str, V>,
+    function: &FunctionCallNode,
+) -> TemplateParseResult<&'a V> {
+    if let Some(value) = table.get(function.name) {
+        Ok(value)
+    } else {
+        let candidates = collect_similar(function.name, table.keys());
+        Err(TemplateParseError::with_span(
+            TemplateParseErrorKind::NoSuchFunction {
+                name: function.name.to_owned(),
+                candidates,
+            },
+            function.name_span,
+        ))
+    }
+}
+
 /// Looks up `table` by the given method name.
 pub fn lookup_method<'a, V>(
     type_name: impl Into<String>,
@@ -1257,7 +1226,7 @@ mod tests {
         );
         assert_matches!(
             parse_into_kind(&format!("{}", (i64::MAX as u64) + 1)),
-            Err(TemplateParseErrorKind::ParseIntError(_))
+            Err(TemplateParseErrorKind::Expression(_))
         );
     }
 
