@@ -21,9 +21,9 @@ use std::{env, fmt, io, mem};
 use minus::Pager as MinusPager;
 use tracing::instrument;
 
-use crate::command_error::CommandError;
+use crate::command_error::{config_error_with_message, CommandError};
 use crate::config::CommandNameAndArgs;
-use crate::formatter::{Formatter, FormatterFactory, LabeledWriter};
+use crate::formatter::{Formatter, FormatterFactory, HeadingLabeledWriter, LabeledWriter};
 
 const BUILTIN_PAGER_NAME: &str = ":builtin";
 
@@ -163,6 +163,7 @@ impl Write for UiStderr<'_> {
 
 pub struct Ui {
     color: bool,
+    quiet: bool,
     pager_cmd: CommandNameAndArgs,
     paginate: PaginationChoice,
     progress_indicator: bool,
@@ -222,6 +223,10 @@ fn use_color(choice: ColorChoice) -> bool {
     }
 }
 
+fn be_quiet(config: &config::Config) -> bool {
+    config.get_bool("ui.quiet").unwrap_or_default()
+}
+
 #[derive(Clone, Copy, Debug, Default, Eq, PartialEq, serde::Deserialize)]
 #[serde(rename_all(deserialize = "kebab-case"))]
 pub enum PaginationChoice {
@@ -233,18 +238,19 @@ pub enum PaginationChoice {
 fn pagination_setting(config: &config::Config) -> Result<PaginationChoice, CommandError> {
     config
         .get::<PaginationChoice>("ui.paginate")
-        .map_err(|err| CommandError::ConfigError(format!("Invalid `ui.paginate`: {err}")))
+        .map_err(|err| config_error_with_message("Invalid `ui.paginate`", err))
 }
 
 fn pager_setting(config: &config::Config) -> Result<CommandNameAndArgs, CommandError> {
     config
         .get::<CommandNameAndArgs>("ui.pager")
-        .map_err(|err| CommandError::ConfigError(format!("Invalid `ui.pager`: {err}")))
+        .map_err(|err| config_error_with_message("Invalid `ui.pager`", err))
 }
 
 impl Ui {
     pub fn with_config(config: &config::Config) -> Result<Ui, CommandError> {
         let color = use_color(color_setting(config));
+        let quiet = be_quiet(config);
         // Sanitize ANSI escape codes if we're printing to a terminal. Doesn't affect
         // ANSI escape codes that originate from the formatter itself.
         let sanitize = io::stdout().is_terminal();
@@ -252,6 +258,7 @@ impl Ui {
         let progress_indicator = progress_indicator_setting(config);
         Ok(Ui {
             color,
+            quiet,
             formatter_factory,
             pager_cmd: pager_setting(config)?,
             paginate: pagination_setting(config)?,
@@ -262,6 +269,7 @@ impl Ui {
 
     pub fn reset(&mut self, config: &config::Config) -> Result<(), CommandError> {
         self.color = use_color(color_setting(config));
+        self.quiet = be_quiet(config);
         self.paginate = pagination_setting(config)?;
         self.pager_cmd = pager_setting(config)?;
         self.progress_indicator = progress_indicator_setting(config);
@@ -292,7 +300,7 @@ impl Ui {
                     Err(e) => {
                         // The pager executable couldn't be found or couldn't be run
                         writeln!(
-                            self.warning(),
+                            self.warning_default(),
                             "Failed to spawn pager '{name}': {e}. Consider using the `:builtin` \
                              pager.",
                             name = self.pager_cmd.split_name(),
@@ -373,18 +381,74 @@ impl Ui {
         })
     }
 
-    pub fn hint(&self) -> LabeledWriter<Box<dyn Formatter + '_>, &'static str> {
-        LabeledWriter::new(self.stderr_formatter(), "hint")
+    /// Writer to print an update that's not part of the command's main output.
+    pub fn status(&self) -> Box<dyn Write + '_> {
+        if self.quiet {
+            Box::new(std::io::sink())
+        } else {
+            Box::new(self.stderr())
+        }
     }
 
-    pub fn warning(&self) -> LabeledWriter<Box<dyn Formatter + '_>, &'static str> {
+    /// A formatter to print an update that's not part of the command's main
+    /// output. Returns `None` if `--quiet` was requested.
+    pub fn status_formatter(&self) -> Option<Box<dyn Formatter + '_>> {
+        (!self.quiet).then(|| self.stderr_formatter())
+    }
+
+    /// Writer to print hint with the default "Hint: " heading.
+    pub fn hint_default(
+        &self,
+    ) -> Option<HeadingLabeledWriter<Box<dyn Formatter + '_>, &'static str, &'static str>> {
+        self.hint_with_heading("Hint: ")
+    }
+
+    /// Writer to print hint without the "Hint: " heading.
+    pub fn hint_no_heading(&self) -> Option<LabeledWriter<Box<dyn Formatter + '_>, &'static str>> {
+        (!self.quiet).then(|| LabeledWriter::new(self.stderr_formatter(), "hint"))
+    }
+
+    /// Writer to print hint with the given heading.
+    pub fn hint_with_heading<H: fmt::Display>(
+        &self,
+        heading: H,
+    ) -> Option<HeadingLabeledWriter<Box<dyn Formatter + '_>, &'static str, H>> {
+        (!self.quiet).then(|| HeadingLabeledWriter::new(self.stderr_formatter(), "hint", heading))
+    }
+
+    /// Writer to print warning with the default "Warning: " heading.
+    pub fn warning_default(
+        &self,
+    ) -> HeadingLabeledWriter<Box<dyn Formatter + '_>, &'static str, &'static str> {
+        self.warning_with_heading("Warning: ")
+    }
+
+    /// Writer to print warning without the "Warning: " heading.
+    pub fn warning_no_heading(&self) -> LabeledWriter<Box<dyn Formatter + '_>, &'static str> {
         LabeledWriter::new(self.stderr_formatter(), "warning")
     }
 
-    pub fn error(&self) -> LabeledWriter<Box<dyn Formatter + '_>, &'static str> {
+    /// Writer to print warning with the given heading.
+    pub fn warning_with_heading<H: fmt::Display>(
+        &self,
+        heading: H,
+    ) -> HeadingLabeledWriter<Box<dyn Formatter + '_>, &'static str, H> {
+        HeadingLabeledWriter::new(self.stderr_formatter(), "warning", heading)
+    }
+
+    /// Writer to print error without the "Error: " heading.
+    pub fn error_no_heading(&self) -> LabeledWriter<Box<dyn Formatter + '_>, &'static str> {
         LabeledWriter::new(self.stderr_formatter(), "error")
     }
 
+    /// Writer to print error with the given heading.
+    pub fn error_with_heading<H: fmt::Display>(
+        &self,
+        heading: H,
+    ) -> HeadingLabeledWriter<Box<dyn Formatter + '_>, &'static str, H> {
+        HeadingLabeledWriter::new(self.stderr_formatter(), "error", heading)
+    }
+
     /// Waits for the pager exits.
     #[instrument(skip_all)]
     pub fn finalize_pager(&mut self) {
@@ -398,7 +462,7 @@ impl Ui {
                     // It's possible (though unlikely) that this write fails, but
                     // this function gets called so late that there's not much we
                     // can do about it.
-                    writeln!(self.error(), "Failed to wait on pager: {e}").ok();
+                    writeln!(self.warning_default(), "Failed to wait on pager: {e}").ok();
                 }
             }
             UiOutput::BuiltinPaged { pager } => {
@@ -415,7 +479,7 @@ impl Ui {
                 .unwrap_or(false)
     }
 
-    pub fn prompt(&mut self, prompt: &str) -> io::Result<String> {
+    pub fn prompt(&self, prompt: &str) -> io::Result<String> {
         if !Self::can_prompt() {
             return Err(io::Error::new(
                 io::ErrorKind::Unsupported,
@@ -441,7 +505,7 @@ impl Ui {
 
     /// Repeat the given prompt until the input is one of the specified choices.
     pub fn prompt_choice(
-        &mut self,
+        &self,
         prompt: &str,
         choices: &[impl AsRef<str>],
         default: Option<&str>,
@@ -465,12 +529,12 @@ impl Ui {
                 return Ok(choice);
             }
 
-            writeln!(self.warning(), "unrecognized response")?;
+            writeln!(self.warning_no_heading(), "unrecognized response")?;
         }
     }
 
     /// Prompts for a yes-or-no response, with yes = true and no = false.
-    pub fn prompt_yes_no(&mut self, prompt: &str, default: Option<bool>) -> io::Result<bool> {
+    pub fn prompt_yes_no(&self, prompt: &str, default: Option<bool>) -> io::Result<bool> {
         let default_str = match &default {
             Some(true) => "(Yn)",
             Some(false) => "(yN)",
@@ -486,7 +550,7 @@ impl Ui {
         Ok(choice.starts_with(['y', 'Y']))
     }
 
-    pub fn prompt_password(&mut self, prompt: &str) -> io::Result<String> {
+    pub fn prompt_password(&self, prompt: &str) -> io::Result<String> {
         if !io::stdout().is_terminal() {
             return Err(io::Error::new(
                 io::ErrorKind::Unsupported,
