@@ -779,14 +779,14 @@ fn test_diff_external_tool() {
 
     insta::assert_snapshot!(
         test_env.jj_cmd_success(&repo_path, &["log", "-p", "--tool=fake-diff-editor"]), @r###"
-    @  rlvkpnrz test.user@example.com 2001-02-03 04:05:09.000 +07:00 0cba70c7
+    @  rlvkpnrz test.user@example.com 2001-02-03 08:05:09 0cba70c7
     │  (no description set)
     │  file1
     │  file2
     │  --
     │  file2
     │  file3
-    ◉  qpvuntsm test.user@example.com 2001-02-03 04:05:08.000 +07:00 39b5a56f
+    ◉  qpvuntsm test.user@example.com 2001-02-03 08:05:08 39b5a56f
     │  (no description set)
     │  --
     │  file1
@@ -799,8 +799,8 @@ fn test_diff_external_tool() {
         test_env.jj_cmd_success(&repo_path, &["show", "--tool=fake-diff-editor"]), @r###"
     Commit ID: 0cba70c72186eabb5a2f91be63a8366b9f6da6c6
     Change ID: rlvkpnrzqnoowoytxnquwvuryrwnrmlp
-    Author: Test User <test.user@example.com> (2001-02-03 04:05:08.000 +07:00)
-    Committer: Test User <test.user@example.com> (2001-02-03 04:05:09.000 +07:00)
+    Author: Test User <test.user@example.com> (2001-02-03 08:05:08)
+    Committer: Test User <test.user@example.com> (2001-02-03 08:05:09)
 
         (no description set)
 
@@ -846,15 +846,15 @@ fn test_diff_external_tool() {
     insta::assert_snapshot!(stdout, @r###"
     Commit ID: 0cba70c72186eabb5a2f91be63a8366b9f6da6c6
     Change ID: rlvkpnrzqnoowoytxnquwvuryrwnrmlp
-    Author: Test User <test.user@example.com> (2001-02-03 04:05:08.000 +07:00)
-    Committer: Test User <test.user@example.com> (2001-02-03 04:05:09.000 +07:00)
+    Author: Test User <test.user@example.com> (2001-02-03 08:05:08)
+    Committer: Test User <test.user@example.com> (2001-02-03 08:05:09)
 
         (no description set)
 
     diff
     "###);
     insta::assert_snapshot!(stderr, @r###"
-    Tool exited with a non-zero code (run with --debug to see the exact invocation). Exit code: 1.
+    Warning: Tool exited with a non-zero code (run with --debug to see the exact invocation). Exit code: 1.
     "###);
 
     // --tool=:builtin shouldn't be ignored
