@@ -92,15 +92,26 @@ fn test_no_subcommand() {
     test_env.jj_cmd_ok(&repo_path, &["branch", "create", "show"]);
     // TODO: test_env.jj_cmd_ok(&repo_path, &["-r", "help"])
     insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["-r", "log"]), @r###"
-    @  qpvuntsm test.user@example.com 2001-02-03 04:05:07.000 +07:00 help log show 230dd059
+    @  qpvuntsm test.user@example.com 2001-02-03 08:05:07 help log show 230dd059
     │  (empty) (no description set)
     ~
     "###);
     insta::assert_snapshot!(test_env.jj_cmd_success(&repo_path, &["-r", "show"]), @r###"
-    @  qpvuntsm test.user@example.com 2001-02-03 04:05:07.000 +07:00 help log show 230dd059
+    @  qpvuntsm test.user@example.com 2001-02-03 08:05:07 help log show 230dd059
     │  (empty) (no description set)
     ~
     "###);
+
+    // Multiple default command strings work.
+    test_env.add_config(r#"ui.default-command=["commit", "-m", "foo"]"#);
+    test_env.jj_cmd_ok(&repo_path, &["new"]);
+    std::fs::write(repo_path.join("file.txt"), "file").unwrap();
+    let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &[]);
+    assert_eq!(stdout, "");
+    insta::assert_snapshot!(stderr, @r###"
+    Working copy now at: kxryzmor 70ac3df3 (empty) (no description set)
+    Parent commit      : lylxulpl 9dbbb452 foo
+    "###);
 }
 
 #[test]
@@ -214,6 +225,25 @@ fn test_no_workspace_directory() {
     "###);
 }
 
+#[test]
+fn test_bad_path() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["cat", "../out"]);
+    insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
+    Error: Path "../out" is not in the repo "."
+    Caused by: Invalid component ".." in repo-relative path "../out"
+    "###);
+
+    let stderr = test_env.jj_cmd_failure(test_env.env_root(), &["cat", "-Rrepo", "out"]);
+    insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
+    Error: Path "out" is not in the repo "repo"
+    Caused by: Invalid component ".." in repo-relative path "../out"
+    "###);
+}
+
 #[test]
 fn test_broken_repo_structure() {
     let test_env = TestEnvironment::default();
@@ -355,6 +385,75 @@ fn test_color_config() {
     "###);
 }
 
+#[test]
+fn test_color_ui_messages() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+    test_env.add_config("ui.color = 'always'");
+
+    // hint and error
+    let stderr = test_env.jj_cmd_failure(test_env.env_root(), &["-R."]);
+    insta::assert_snapshot!(stderr, @r###"
+    [1m[38;5;6mHint: [0m[39mUse `jj -h` for a list of available commands.[39m
+    [39mRun `jj config set --user ui.default-command log` to disable this message.[39m
+    [1m[38;5;1mError: [39mThere is no jj repo in "."[0m
+    "###);
+
+    // error source
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["log", ".."]);
+    insta::assert_snapshot!(stderr.replace('\\', "/"), @r###"
+    [1m[38;5;1mError: [39mPath ".." is not in the repo "."[0m
+    [1m[39mCaused by: [0m[39mInvalid component ".." in repo-relative path "../"[39m
+    "###);
+
+    // warning
+    let (_stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["log", "@"]);
+    insta::assert_snapshot!(stderr, @r###"
+    [1m[38;5;3mWarning: [39mThe argument "@" is being interpreted as a path. To specify a revset, pass -r "@" instead.[0m
+    "###);
+
+    // error inlined in template output
+    test_env.jj_cmd_ok(&repo_path, &["new"]);
+    let stdout = test_env.jj_cmd_success(
+        &repo_path,
+        &[
+            "log",
+            "-r@|@--",
+            "--config-toml=templates.log_node='commit_id'",
+            "-Tdescription",
+        ],
+    );
+    insta::assert_snapshot!(stdout, @r###"
+    [38;5;4m167f90e7600a50f85c4f909b53eaf546faa82879[39m
+    [1m[39m<[38;5;1mError: [39mNo commit available>[0m  [38;5;8m(elided revisions)[39m
+    [38;5;4m0000000000000000000000000000000000000000[39m
+    "###);
+
+    // formatted hint
+    let stderr = test_env.jj_cmd_failure(&repo_path, &["new", ".."]);
+    insta::assert_snapshot!(stderr, @r###"
+    [1m[38;5;1mError: [39mRevset ".." resolved to more than one revision[0m
+    [1m[38;5;6mHint: [0m[39mThe revset ".." resolved to these revisions:[39m
+    [39m  [1m[38;5;5mm[0m[38;5;8mzvwutvl[39m [1m[38;5;4m1[0m[38;5;8m67f90e7[39m [38;5;2m(empty)[39m [38;5;2m(no description set)[39m[39m
+    [39m  [1m[38;5;5mq[0m[38;5;8mpvuntsm[39m [1m[38;5;4m2[0m[38;5;8m30dd059[39m [38;5;2m(empty)[39m [38;5;2m(no description set)[39m[39m
+    [1m[38;5;6mHint: [0m[39mPrefix the expression with 'all:' to allow any number of revisions (i.e. 'all:..').[39m
+    "###);
+}
+
+#[test]
+fn test_quiet() {
+    let test_env = TestEnvironment::default();
+    test_env.jj_cmd_ok(test_env.env_root(), &["git", "init", "repo"]);
+    let repo_path = test_env.env_root().join("repo");
+
+    // Can skip message about new working copy with `--quiet`
+    std::fs::write(repo_path.join("file1"), "contents").unwrap();
+    let (_stdout, stderr) =
+        test_env.jj_cmd_ok(&repo_path, &["--quiet", "describe", "-m=new description"]);
+    insta::assert_snapshot!(stderr, @"");
+}
+
 #[test]
 fn test_early_args() {
     // Test that help output parses early args
@@ -406,7 +505,7 @@ fn test_no_user_configured() {
     insta::assert_snapshot!(get_stderr_string(&assert), @r###"
     Working copy now at: qpvuntsm 7a7d6016 (empty) without name
     Parent commit      : zzzzzzzz 00000000 (empty) (no description set)
-    Name and email not configured. Until configured, your commits will be created with the empty identity, and can't be pushed to remotes. To configure, run:
+    Warning: Name and email not configured. Until configured, your commits will be created with the empty identity, and can't be pushed to remotes. To configure, run:
       jj config set --user user.name "Some One"
       jj config set --user user.email "someone@example.com"
     "###);
@@ -418,7 +517,7 @@ fn test_no_user_configured() {
     insta::assert_snapshot!(get_stderr_string(&assert), @r###"
     Working copy now at: qpvuntsm 906f8b89 (empty) without email
     Parent commit      : zzzzzzzz 00000000 (empty) (no description set)
-    Name and email not configured. Until configured, your commits will be created with the empty identity, and can't be pushed to remotes. To configure, run:
+    Warning: Name and email not configured. Until configured, your commits will be created with the empty identity, and can't be pushed to remotes. To configure, run:
       jj config set --user user.name "Some One"
       jj config set --user user.email "someone@example.com"
     "###);
@@ -449,6 +548,7 @@ fn test_help() {
           --at-operation <AT_OPERATION>  Operation to load the repo at [default: @] [aliases: at-op]
           --debug                        Enable debug logging
           --color <WHEN>                 When to colorize output (always, never, auto)
+          --quiet                        Silence non-primary command output
           --no-pager                     Disable the pager
           --config-toml <TOML>           Additional configuration options (can be repeated)
     "###);
