@@ -14,6 +14,8 @@
 
 use std::path::Path;
 
+use indoc::indoc;
+
 use crate::common::TestEnvironment;
 
 fn create_commit(
@@ -195,15 +197,16 @@ fn test_resolution() {
         &editor_script,
         [
             "dump editor2",
-            "write
-<<<<<<<
-%%%%%%%
--some
-+fake
-+++++++
-conflict
->>>>>>>
-",
+            indoc! {"
+                write
+                <<<<<<<
+                %%%%%%%
+                -some
+                +fake
+                +++++++
+                conflict
+                >>>>>>>
+            "},
         ]
         .join("\0"),
     )
@@ -220,13 +223,13 @@ conflict
     insta::assert_snapshot!(stderr, @r###"
     Resolving conflicts in: file
     New conflicts appeared in these commits:
-      vruxwmqv 23991847 conflict | (conflict) conflict
+      vruxwmqv 8144e92d conflict | (conflict) conflict
     To resolve the conflicts, start by updating to it:
       jj new vruxwmqvtpmx
     Then use `jj resolve`, or edit the conflict markers in the file directly.
     Once the conflicts are resolved, you may want inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
-    Working copy now at: vruxwmqv 23991847 conflict | (conflict) conflict
+    Working copy now at: vruxwmqv 8144e92d conflict | (conflict) conflict
     Parent commit      : zsuskuln aa493daf a | a
     Parent commit      : royxmykx db6a4daf b | b
     Added 0 files, modified 1 files, removed 0 files
@@ -270,15 +273,16 @@ conflict
         &editor_script,
         [
             "dump editor3",
-            "write
-<<<<<<<
-%%%%%%%
--some
-+fake
-+++++++
-conflict
->>>>>>>
-",
+            indoc! {"
+                write
+                <<<<<<<
+                %%%%%%%
+                -some
+                +fake
+                +++++++
+                conflict
+                >>>>>>>
+            "},
         ]
         .join("\0"),
     )
@@ -443,7 +447,7 @@ fn test_too_many_parents() {
 
     let error = test_env.jj_cmd_failure(&repo_path, &["resolve"]);
     insta::assert_snapshot!(error, @r###"
-    Using default editor ':builtin'; run `jj config set --user ui.merge-editor :builtin` to disable this message.
+    Hint: Using default editor ':builtin'; run `jj config set --user ui.merge-editor :builtin` to disable this message.
     Resolving conflicts in: file
     Error: Failed to resolve conflicts
     Caused by: The conflict at "file" has 3 sides. At most 2 sides are supported.
@@ -521,7 +525,7 @@ fn test_file_vs_dir() {
     "###);
     let error = test_env.jj_cmd_failure(&repo_path, &["resolve"]);
     insta::assert_snapshot!(error, @r###"
-    Using default editor ':builtin'; run `jj config set --user ui.merge-editor :builtin` to disable this message.
+    Hint: Using default editor ':builtin'; run `jj config set --user ui.merge-editor :builtin` to disable this message.
     Resolving conflicts in: file
     Error: Failed to resolve conflicts
     Caused by: Only conflicts that involve normal files (not symlinks, not executable, etc.) are supported. Conflict summary for "file":
@@ -578,7 +582,7 @@ fn test_description_with_dir_and_deletion() {
     "###);
     let error = test_env.jj_cmd_failure(&repo_path, &["resolve"]);
     insta::assert_snapshot!(error, @r###"
-    Using default editor ':builtin'; run `jj config set --user ui.merge-editor :builtin` to disable this message.
+    Hint: Using default editor ':builtin'; run `jj config set --user ui.merge-editor :builtin` to disable this message.
     Resolving conflicts in: file
     Error: Failed to resolve conflicts
     Caused by: Only conflicts that involve normal files (not symlinks, not executable, etc.) are supported. Conflict summary for "file":
@@ -690,13 +694,13 @@ fn test_multiple_conflicts() {
     insta::assert_snapshot!(stderr, @r###"
     Resolving conflicts in: another_file
     New conflicts appeared in these commits:
-      vruxwmqv c3c25bce conflict | (conflict) conflict
+      vruxwmqv 1e22a8e4 conflict | (conflict) conflict
     To resolve the conflicts, start by updating to it:
       jj new vruxwmqvtpmx
     Then use `jj resolve`, or edit the conflict markers in the file directly.
     Once the conflicts are resolved, you may want inspect the result with `jj diff`.
     Then run `jj squash` to move the resolution into the conflicted commit.
-    Working copy now at: vruxwmqv c3c25bce conflict | (conflict) conflict
+    Working copy now at: vruxwmqv 1e22a8e4 conflict | (conflict) conflict
     Parent commit      : zsuskuln de7553ef a | a
     Parent commit      : royxmykx f68bc2f0 b | b
     Added 0 files, modified 1 files, removed 0 files
@@ -724,20 +728,7 @@ fn test_multiple_conflicts() {
     std::fs::write(&editor_script, "expect\n\0write\nresolution another_file\n").unwrap();
     let (stdout, stderr) = test_env.jj_cmd_ok(&repo_path, &["resolve", "--quiet", "another_file"]);
     insta::assert_snapshot!(stdout, @"");
-    insta::assert_snapshot!(stderr, @r###"
-    Resolving conflicts in: another_file
-    New conflicts appeared in these commits:
-      vruxwmqv fd3874cd conflict | (conflict) conflict
-    To resolve the conflicts, start by updating to it:
-      jj new vruxwmqvtpmx
-    Then use `jj resolve`, or edit the conflict markers in the file directly.
-    Once the conflicts are resolved, you may want inspect the result with `jj diff`.
-    Then run `jj squash` to move the resolution into the conflicted commit.
-    Working copy now at: vruxwmqv fd3874cd conflict | (conflict) conflict
-    Parent commit      : zsuskuln de7553ef a | a
-    Parent commit      : royxmykx f68bc2f0 b | b
-    Added 0 files, modified 1 files, removed 0 files
-    "###);
+    insta::assert_snapshot!(stderr, @"");
 
     // For the rest of the test, we call `jj resolve` several times in a row to
     // resolve each conflict in the order it chooses.
