@@ -30,18 +30,14 @@ use crate::matchers::{Matcher, Visit};
 use crate::merged_tree::{MergedTree, MergedTreeBuilder};
 use crate::object_id::ObjectId;
 use crate::op_store::RefTarget;
-use crate::repo::{MutableRepo, Repo};
+use crate::repo::{MutableRepo, Repo, RewriteType};
 use crate::repo_path::RepoPath;
 use crate::revset::{RevsetExpression, RevsetIteratorExt};
 use crate::settings::UserSettings;
 use crate::store::Store;
-use crate::tree::TreeMergeError;
 
 #[instrument(skip(repo))]
-pub fn merge_commit_trees(
-    repo: &dyn Repo,
-    commits: &[Commit],
-) -> Result<MergedTree, TreeMergeError> {
+pub fn merge_commit_trees(repo: &dyn Repo, commits: &[Commit]) -> BackendResult<MergedTree> {
     merge_commit_trees_without_repo(repo.store(), repo.index(), commits)
 }
 
@@ -50,7 +46,7 @@ pub fn merge_commit_trees_without_repo(
     store: &Arc<Store>,
     index: &dyn Index,
     commits: &[Commit],
-) -> Result<MergedTree, TreeMergeError> {
+) -> BackendResult<MergedTree> {
     if commits.is_empty() {
         Ok(store.get_root_tree(&store.empty_merged_tree_id())?)
     } else {
@@ -104,7 +100,7 @@ pub fn rebase_commit(
     mut_repo: &mut MutableRepo,
     old_commit: &Commit,
     new_parents: &[Commit],
-) -> Result<Commit, TreeMergeError> {
+) -> BackendResult<Commit> {
     let rebased_commit = rebase_commit_with_options(
         settings,
         mut_repo,
@@ -129,7 +125,26 @@ pub fn rebase_commit_with_options(
     old_commit: &Commit,
     new_parents: &[Commit],
     options: &RebaseOptions,
-) -> Result<RebasedCommit, TreeMergeError> {
+) -> BackendResult<RebasedCommit> {
+    // If specified, don't create commit where one parent is an ancestor of another.
+    let simplified_new_parents;
+    let new_parents = if options.simplify_ancestor_merge {
+        let mut new_parent_ids = new_parents.iter().map(|commit| commit.id());
+        let head_set: HashSet<_> = mut_repo
+            .index()
+            .heads(&mut new_parent_ids)
+            .into_iter()
+            .collect();
+        simplified_new_parents = new_parents
+            .iter()
+            .filter(|commit| head_set.contains(commit.id()))
+            .cloned()
+            .collect_vec();
+        &simplified_new_parents[..]
+    } else {
+        new_parents
+    };
+
     let old_parents = old_commit.parents();
     let old_parent_trees = old_parents
         .iter()
@@ -169,10 +184,10 @@ pub fn rebase_commit_with_options(
             EmptyBehaviour::AbandonAllEmpty => *parent.tree_id() == new_tree_id,
         };
         if should_abandon {
-            // Record old_commit as being succeeded by the parent.
-            // This ensures that when we stack commits, the second commit knows to
-            // rebase on top of the parent commit, rather than the abandoned commit.
-            mut_repo.record_rewritten_commit(old_commit.id().clone(), parent.id().clone());
+            mut_repo.record_abandoned_commit_with_parents(
+                old_commit.id().clone(),
+                std::iter::once(parent.id().clone()),
+            );
             return Ok(RebasedCommit::Abandoned {
                 parent: parent.clone(),
             });
@@ -194,7 +209,7 @@ pub fn rebase_to_dest_parent(
     repo: &dyn Repo,
     source: &Commit,
     destination: &Commit,
-) -> Result<MergedTree, TreeMergeError> {
+) -> BackendResult<MergedTree> {
     if source.parent_ids() == destination.parent_ids() {
         Ok(source.tree()?)
     } else {
@@ -211,7 +226,7 @@ pub fn back_out_commit(
     mut_repo: &mut MutableRepo,
     old_commit: &Commit,
     new_parents: &[Commit],
-) -> Result<Commit, TreeMergeError> {
+) -> BackendResult<Commit> {
     let old_base_tree = merge_commit_trees(mut_repo, &old_commit.parents())?;
     let new_base_tree = merge_commit_trees(mut_repo, new_parents)?;
     let old_tree = old_commit.tree()?;
@@ -221,10 +236,10 @@ pub fn back_out_commit(
         .map(|commit| commit.id().clone())
         .collect();
     // TODO: i18n the description based on repo language
-    Ok(mut_repo
+    mut_repo
         .new_commit(settings, new_parent_ids, new_tree.id())
         .set_description(format!("backout of commit {}", &old_commit.id().hex()))
-        .write()?)
+        .write()
 }
 
 #[derive(Clone, Default, PartialEq, Eq, Debug)]
@@ -259,27 +274,9 @@ pub struct RebaseOptions {
 pub(crate) struct DescendantRebaser<'settings, 'repo> {
     settings: &'settings UserSettings,
     mut_repo: &'repo mut MutableRepo,
-    // The commit identified by the key has been replaced by all the ones in the value, typically
-    // because the key commit was abandoned (the value commits are then the abandoned commit's
-    // parents). A child of the key commit should be rebased onto all the value commits. A branch
-    // pointing to the key commit should become a conflict pointing to all the value commits.
-    parent_mapping: HashMap<CommitId, Vec<CommitId>>,
-    divergent: HashMap<CommitId, Vec<CommitId>>,
     // In reverse order (parents after children), so we can remove the last one to rebase first.
     to_visit: Vec<Commit>,
-    // Commits to visit but skip. These were also in `to_visit` to start with, but we don't
-    // want to rebase them. Instead, we record them in `parent_mapping` when we visit them. That
-    // way, their descendants will be rebased correctly.
-    abandoned: HashSet<CommitId>,
-    new_commits: HashSet<CommitId>,
     rebased: HashMap<CommitId, CommitId>,
-    // Names of branches where local target includes the commit id in the key.
-    branches: HashMap<CommitId, HashSet<String>>,
-    // Parents of rebased/abandoned commit that should become new heads once their descendants
-    // have been rebased.
-    heads_to_add: HashSet<CommitId>,
-    heads_to_remove: Vec<CommitId>,
-
     // Options to apply during a rebase.
     options: RebaseOptions,
 }
@@ -294,27 +291,13 @@ impl<'settings, 'repo> DescendantRebaser<'settings, 'repo> {
     pub fn new(
         settings: &'settings UserSettings,
         mut_repo: &'repo mut MutableRepo,
-        rewritten: HashMap<CommitId, HashSet<CommitId>>,
-        abandoned: HashSet<CommitId>,
     ) -> DescendantRebaser<'settings, 'repo> {
         let store = mut_repo.store();
-        let root_commit_id = store.root_commit_id();
-        assert!(!abandoned.contains(root_commit_id));
-        assert!(!rewritten.contains_key(root_commit_id));
-        let old_commits_expression = RevsetExpression::commits(rewritten.keys().cloned().collect())
-            .union(&RevsetExpression::commits(
-                abandoned.iter().cloned().collect(),
-            ));
-        let heads_to_add_expression = old_commits_expression
-            .parents()
+        let old_commits_expression =
+            RevsetExpression::commits(mut_repo.parent_mapping.keys().cloned().collect());
+        let to_visit_expression = old_commits_expression
+            .descendants()
             .minus(&old_commits_expression);
-        let heads_to_add = heads_to_add_expression
-            .evaluate_programmatic(mut_repo)
-            .unwrap()
-            .iter()
-            .collect();
-
-        let to_visit_expression = old_commits_expression.descendants();
         let to_visit_revset = to_visit_expression.evaluate_programmatic(mut_repo).unwrap();
         let to_visit: Vec<_> = to_visit_revset.iter().commits(store).try_collect().unwrap();
         drop(to_visit_revset);
@@ -330,7 +313,7 @@ impl<'settings, 'repo> DescendantRebaser<'settings, 'repo> {
                 visited.insert(commit.id().clone());
                 let mut dependents = vec![];
                 for parent in commit.parents() {
-                    if let Some(targets) = rewritten.get(parent.id()) {
+                    if let Some((_, targets)) = mut_repo.parent_mapping.get(parent.id()) {
                         for target in targets {
                             if to_visit_set.contains(target) && !visited.contains(target) {
                                 dependents.push(store.get_commit(target).unwrap());
@@ -345,44 +328,11 @@ impl<'settings, 'repo> DescendantRebaser<'settings, 'repo> {
             },
         );
 
-        let new_commits = rewritten.values().flatten().cloned().collect();
-
-        let mut parent_mapping = HashMap::new();
-        let mut divergent = HashMap::new();
-        for (old_commit, new_commits) in rewritten {
-            if new_commits.len() == 1 {
-                parent_mapping.insert(old_commit, vec![new_commits.iter().next().unwrap().clone()]);
-            } else {
-                // The call to index.heads() is mostly to get a predictable order
-                let new_commits = mut_repo.index().heads(&mut new_commits.iter());
-                divergent.insert(old_commit, new_commits);
-            }
-        }
-
-        // Build a map from commit to branches pointing to it, so we don't need to scan
-        // all branches each time we rebase a commit.
-        let mut branches: HashMap<_, HashSet<_>> = HashMap::new();
-        for (branch_name, target) in mut_repo.view().local_branches() {
-            for commit in target.added_ids() {
-                branches
-                    .entry(commit.clone())
-                    .or_default()
-                    .insert(branch_name.to_owned());
-            }
-        }
-
         DescendantRebaser {
             settings,
             mut_repo,
-            parent_mapping,
-            divergent,
             to_visit,
-            abandoned,
-            new_commits,
             rebased: Default::default(),
-            branches,
-            heads_to_add,
-            heads_to_remove: Default::default(),
             options: Default::default(),
         }
     }
@@ -399,60 +349,6 @@ impl<'settings, 'repo> DescendantRebaser<'settings, 'repo> {
         self.rebased
     }
 
-    /// Panics if `parent_mapping` contains cycles
-    fn new_parents(&self, old_ids: &[CommitId]) -> Vec<CommitId> {
-        fn single_substitution_round(
-            parent_mapping: &HashMap<CommitId, Vec<CommitId>>,
-            ids: Vec<CommitId>,
-        ) -> (Vec<CommitId>, bool) {
-            let mut made_replacements = false;
-            let mut new_ids = vec![];
-            // TODO(ilyagr): (Maybe?) optimize common case of replacements all
-            // being singletons. If CommitId-s were Copy. no allocations would be needed in
-            // that case, but it probably doesn't matter much while they are Vec<u8>-s.
-            for id in ids.into_iter() {
-                match parent_mapping.get(&id) {
-                    None => new_ids.push(id),
-                    Some(replacements) => {
-                        assert!(
-                            // Each commit must have a parent, so a parent can
-                            // not just be mapped to nothing. This assertion
-                            // could be removed if this function is used for
-                            // mapping something other than a commit's parents.
-                            !replacements.is_empty(),
-                            "Found empty value for key {id:?} in the parent mapping",
-                        );
-                        made_replacements = true;
-                        new_ids.extend(replacements.iter().cloned())
-                    }
-                };
-            }
-            (new_ids, made_replacements)
-        }
-
-        let mut new_ids: Vec<CommitId> = old_ids.to_vec();
-        // The longest possible non-cycle substitution sequence goes through each key of
-        // parent_mapping once.
-        let mut allowed_iterations = 0..self.parent_mapping.len();
-        loop {
-            let made_replacements;
-            (new_ids, made_replacements) = single_substitution_round(&self.parent_mapping, new_ids);
-            if !made_replacements {
-                break;
-            }
-            allowed_iterations
-                .next()
-                .expect("cycle detected in the parent mapping");
-        }
-        match new_ids.as_slice() {
-            // The first two cases are an optimization for the common case of commits with <=2
-            // parents
-            [_singleton] => new_ids,
-            [a, b] if a != b => new_ids,
-            _ => new_ids.into_iter().unique().collect(),
-        }
-    }
-
     fn ref_target_update(old_id: CommitId, new_ids: Vec<CommitId>) -> (RefTarget, RefTarget) {
         let old_ids = std::iter::repeat(old_id).take(new_ids.len());
         (
@@ -467,18 +363,29 @@ impl<'settings, 'repo> DescendantRebaser<'settings, 'repo> {
         new_commit_ids: Vec<CommitId>,
     ) -> Result<(), BackendError> {
         // We arbitrarily pick a new working-copy commit among the candidates.
-        let abandoned_old_commit = self.abandoned.contains(&old_commit_id);
+        let abandoned_old_commit = matches!(
+            self.mut_repo.parent_mapping.get(&old_commit_id),
+            Some((RewriteType::Abandoned, _))
+        );
         self.update_wc_commits(&old_commit_id, &new_commit_ids[0], abandoned_old_commit)?;
 
-        if let Some(branch_names) = self.branches.get(&old_commit_id).cloned() {
+        // Build a map from commit to branches pointing to it, so we don't need to scan
+        // all branches each time we rebase a commit.
+        // TODO: We no longer need to do this now that we update branches for all
+        // commits at once.
+        let mut branches: HashMap<_, HashSet<_>> = HashMap::new();
+        for (branch_name, target) in self.mut_repo.view().local_branches() {
+            for commit in target.added_ids() {
+                branches
+                    .entry(commit.clone())
+                    .or_default()
+                    .insert(branch_name.to_owned());
+            }
+        }
+
+        if let Some(branch_names) = branches.get(&old_commit_id).cloned() {
             let mut branch_updates = vec![];
             for branch_name in &branch_names {
-                for new_commit_id in &new_commit_ids {
-                    self.branches
-                        .entry(new_commit_id.clone())
-                        .or_default()
-                        .insert(branch_name.clone());
-                }
                 let local_target = self.mut_repo.get_local_branch(branch_name);
                 for old_add in local_target.added_ids() {
                     if *old_add == old_commit_id {
@@ -487,17 +394,13 @@ impl<'settings, 'repo> DescendantRebaser<'settings, 'repo> {
                 }
             }
             let (old_target, new_target) =
-                DescendantRebaser::ref_target_update(old_commit_id.clone(), new_commit_ids);
+                DescendantRebaser::ref_target_update(old_commit_id, new_commit_ids);
             for branch_name in &branch_updates {
                 self.mut_repo
                     .merge_local_branch(branch_name, &old_target, &new_target);
             }
         }
 
-        self.heads_to_add.remove(&old_commit_id);
-        if !self.new_commits.contains(&old_commit_id) || self.rebased.contains_key(&old_commit_id) {
-            self.heads_to_remove.push(old_commit_id);
-        }
         Ok(())
     }
 
@@ -533,45 +436,24 @@ impl<'settings, 'repo> DescendantRebaser<'settings, 'repo> {
         Ok(())
     }
 
-    fn rebase_one(&mut self, old_commit: Commit) -> Result<(), TreeMergeError> {
+    fn rebase_one(&mut self, old_commit: Commit) -> BackendResult<()> {
         let old_commit_id = old_commit.id().clone();
-        if let Some(new_parent_ids) = self.parent_mapping.get(&old_commit_id).cloned() {
-            // This is a commit that had already been rebased before `self` was created
-            // (i.e. it's part of the input for this rebase). We don't need
-            // to rebase it, but we still want to update branches pointing
-            // to the old commit.
-            self.update_references(old_commit_id, new_parent_ids)?;
-            return Ok(());
-        }
-        if let Some(divergent_ids) = self.divergent.get(&old_commit_id).cloned() {
-            // Leave divergent commits in place. Don't update `parent_mapping` since we
-            // don't want to rebase descendants either.
-            self.update_references(old_commit_id, divergent_ids)?;
-            return Ok(());
-        }
+        assert!(!self.mut_repo.parent_mapping.contains_key(&old_commit_id));
         let old_parent_ids = old_commit.parent_ids();
-        let mut new_parent_ids = self.new_parents(old_parent_ids);
-        if self.abandoned.contains(&old_commit_id) {
-            // Update the `new_parents` map so descendants are rebased correctly.
-            self.parent_mapping
-                .insert(old_commit_id.clone(), new_parent_ids.clone());
-            self.update_references(old_commit_id, new_parent_ids)?;
-            return Ok(());
-        } else if new_parent_ids == old_parent_ids {
+        let new_parent_ids = self.mut_repo.new_parents(old_parent_ids);
+        if new_parent_ids == old_parent_ids {
             // The commit is already in place.
             return Ok(());
         }
+        assert_eq!(
+            (
+                self.rebased.get(&old_commit_id),
+                self.mut_repo.parent_mapping.get(&old_commit_id)
+            ),
+            (None, None),
+            "Trying to rebase the same commit {old_commit_id:?} in two different ways",
+        );
 
-        // If specified, don't create commit where one parent is an ancestor of another.
-        if self.options.simplify_ancestor_merge {
-            let head_set: HashSet<_> = self
-                .mut_repo
-                .index()
-                .heads(&mut new_parent_ids.iter())
-                .into_iter()
-                .collect();
-            new_parent_ids.retain(|new_parent| head_set.contains(new_parent));
-        }
         let new_parents: Vec<_> = new_parent_ids
             .iter()
             .map(|new_parent_id| self.mut_repo.store().get_commit(new_parent_id))
@@ -585,40 +467,51 @@ impl<'settings, 'repo> DescendantRebaser<'settings, 'repo> {
         )?;
         let new_commit = match rebased_commit {
             RebasedCommit::Rewritten(new_commit) => new_commit,
-            RebasedCommit::Abandoned { parent } => {
-                self.abandoned.insert(old_commit.id().clone());
-                parent
-            }
+            RebasedCommit::Abandoned { parent } => parent,
         };
-        let previous_rebased_value = self
-            .rebased
+        self.rebased
             .insert(old_commit_id.clone(), new_commit.id().clone());
-        let previous_mapping_value = self
-            .parent_mapping
-            .insert(old_commit_id.clone(), vec![new_commit.id().clone()]);
-        assert_eq!(
-            (previous_rebased_value, previous_mapping_value),
-            (None, None),
-            "Trying to rebase the same commit {old_commit_id:?} in two different ways",
-        );
-        self.update_references(old_commit_id, vec![new_commit.id().clone()])?;
         Ok(())
     }
 
-    pub fn rebase_all(&mut self) -> Result<(), TreeMergeError> {
+    fn update_all_references(&mut self) -> Result<(), BackendError> {
+        for (old_parent_id, (_, new_parent_ids)) in self.mut_repo.parent_mapping.clone() {
+            // Call `new_parents()` here since `parent_mapping` only contains direct
+            // mappings, not transitive ones.
+            // TODO: keep parent_mapping updated with transitive mappings so we don't need
+            // to call `new_parents()` here.
+            let new_parent_ids = self.mut_repo.new_parents(&new_parent_ids);
+            self.update_references(old_parent_id, new_parent_ids)?;
+        }
+        Ok(())
+    }
+
+    fn update_heads(&mut self) {
+        let old_commits_expression =
+            RevsetExpression::commits(self.mut_repo.parent_mapping.keys().cloned().collect());
+        let heads_to_add_expression = old_commits_expression
+            .parents()
+            .minus(&old_commits_expression);
+        let heads_to_add = heads_to_add_expression
+            .evaluate_programmatic(self.mut_repo)
+            .unwrap()
+            .iter();
+
+        let mut view = self.mut_repo.view().store_view().clone();
+        for commit_id in self.mut_repo.parent_mapping.keys() {
+            view.head_ids.remove(commit_id);
+        }
+        view.head_ids.extend(heads_to_add);
+        self.mut_repo.set_view(view);
+    }
+
+    pub fn rebase_all(&mut self) -> BackendResult<()> {
         while let Some(old_commit) = self.to_visit.pop() {
             self.rebase_one(old_commit)?;
         }
-        let mut view = self.mut_repo.view().store_view().clone();
-        for commit_id in &self.heads_to_remove {
-            view.head_ids.remove(commit_id);
-        }
-        for commit_id in &self.heads_to_add {
-            view.head_ids.insert(commit_id.clone());
-        }
-        self.heads_to_remove.clear();
-        self.heads_to_add.clear();
-        self.mut_repo.set_view(view);
+        self.update_all_references()?;
+        self.update_heads();
+
         Ok(())
     }
 }
