@@ -371,6 +371,41 @@ fn test_resolve_working_copy() {
     assert_eq!(resolve(ws2), vec![commit2.id().clone()]);
 }
 
+#[test]
+fn test_resolve_working_copies() {
+    let settings = testutils::user_settings();
+    let test_repo = TestRepo::init();
+    let repo = &test_repo.repo;
+
+    let mut tx = repo.start_transaction(&settings);
+    let mut_repo = tx.mut_repo();
+
+    let commit1 = write_random_commit(mut_repo, &settings);
+    let commit2 = write_random_commit(mut_repo, &settings);
+
+    // Add some workspaces
+    let ws1 = WorkspaceId::new("ws1".to_string());
+    let ws2 = WorkspaceId::new("ws2".to_string());
+
+    // add one commit to each working copy
+    mut_repo
+        .set_wc_commit(ws1.clone(), commit1.id().clone())
+        .unwrap();
+    mut_repo
+        .set_wc_commit(ws2.clone(), commit2.id().clone())
+        .unwrap();
+    let resolve = || -> Vec<CommitId> {
+        RevsetExpression::working_copies()
+            .evaluate_programmatic(mut_repo)
+            .unwrap()
+            .iter()
+            .collect()
+    };
+
+    // ensure our output has those two commits
+    assert_eq!(resolve(), vec![commit2.id().clone(), commit1.id().clone()]);
+}
+
 #[test]
 fn test_resolve_symbol_branches() {
     let settings = testutils::user_settings();
@@ -1279,6 +1314,15 @@ fn test_evaluate_expression_range() {
         ]
     );
 
+    // Range including merge ancestors: commit4-- == root | commit2
+    assert_eq!(
+        resolve_commit_ids(
+            mut_repo,
+            &format!("{}--..{}", commit4.id().hex(), commit3.id().hex())
+        ),
+        vec![commit3.id().clone()]
+    );
+
     // Sibling commits
     assert_eq!(
         resolve_commit_ids(
@@ -2797,3 +2841,26 @@ fn test_no_such_revision_suggestion() {
         if name == "bax" && candidates == vec!["bar".to_string(), "baz".to_string()]
     );
 }
+
+#[test]
+fn test_revset_containing_fn() {
+    let settings = testutils::user_settings();
+    let test_repo = TestRepo::init();
+    let repo = &test_repo.repo;
+
+    let mut tx = repo.start_transaction(&settings);
+    let mut_repo = tx.mut_repo();
+    let commit_a = write_random_commit(mut_repo, &settings);
+    let commit_b = write_random_commit(mut_repo, &settings);
+    let commit_c = write_random_commit(mut_repo, &settings);
+    let commit_d = write_random_commit(mut_repo, &settings);
+    let repo = tx.commit("test");
+
+    let revset = revset_for_commits(repo.as_ref(), &[&commit_b, &commit_d]);
+
+    let revset_has_commit = revset.containing_fn();
+    assert!(!revset_has_commit(commit_a.id()));
+    assert!(revset_has_commit(commit_b.id()));
+    assert!(!revset_has_commit(commit_c.id()));
+    assert!(revset_has_commit(commit_d.id()));
+}
